/*
	$Id: PlateEventTransfer.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Hakkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Plate;
import net.sf.basedb.core.PlateEvent;
import net.sf.basedb.core.PlateEventType;
import net.sf.basedb.core.Protocol;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>PlateEvent</dd>
</dl>

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/

public class PlateEventTransfer extends Transfer
{
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.PlateEventTransfer");
	
	private MappedPlateTransfer mappedPlateT = null;
	private PlateEventTypeTransfer plateEventTypeT = null;
	private ProtocolTransfer protocolT = null;

	/**
	Constructs a new PlateEventTransfer.
	
	@param mappedPlateT A finished {@link MappedPlateTransfer}
	@param plateEventTypeT A finished {@link PlateEventTypeTransfer}
	@param protocolT A finished {@link ProtocolTransfer}
	 */
	PlateEventTransfer(MappedPlateTransfer mappedPlateT,
			PlateEventTypeTransfer plateEventTypeT, ProtocolTransfer protocolT)
	{
		log.debug("Checking that necessary transfers have been done before proceeding with PlateEvent");
		if (mappedPlateT.isDone() && plateEventTypeT.isDone()
				&& protocolT.isDone())
		{
			this.mappedPlateT = mappedPlateT;
			this.plateEventTypeT = plateEventTypeT;
			this.protocolT = protocolT;
		}
		else
		{
			if (!mappedPlateT.isDone()) log.debug("Dependency: Mapped plates haven't been transfered");
			if (!plateEventTypeT.isDone()) log.debug("Dependency: Plate event types haven't been transfered");
			if (!protocolT.isDone()) log.debug("Dependency: Protocols haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring PlateEvents...");
		int count = getInt("countPlateEvents");
		initIdMap(count);
		startProgress(count, "PlateEvents");
		run(prepareStatementFromFile("selectPlateEvents"));
		done();
		log.info("Transferring PlateEvents...DONE");
	}

	/**
	Creates a {@link PlateEvent} from the current position in
	<code>ResultSet</code>.
	
	@param dc {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return <code>PlateEvent</code>
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	PlateEvent createItem(DbControl dc, ResultSet rs) throws BaseException,
			SQLException
	{
		PlateEvent plateEvent = null;
		Integer plateId = mappedPlateT.getBase2Id(rs.getInt(1));
		Integer plateEventTypeId = plateEventTypeT.getBase2Id(rs.getInt(2));
		Integer protocolId = protocolT.getBase2Id(rs.getInt(4));
		if (plateId != null && plateEventTypeId != null)
		{
			Plate plate = Plate.getById(dc, plateId);
			plateEvent = plate.getEvent(PlateEventType.getById(
					dc, plateEventTypeId));
			if (protocolId != null)
			{
				plateEvent.setProtocol(Protocol.getById(dc, protocolId));
			}
			try
			{
				plateEvent.setEventDate(rs.getDate(3));
			}
			catch (SQLException e)
			{
				plateEvent.setEventDate(null);
				log.warn("Couldn't set event date");
				if (log.isDebugEnabled())log.debug(e);
			}
		}
		else
		{
			log.error("Failed: plateId = " + plateId + ", plateEventTypeId = " + plateEventTypeId);
		}
		if (log.isDebugEnabled())
		{
			log.debug("Transfered PlateEvent: " + plateEvent);
			log.debug("--------------------------------------------------------------------------");
		}
		return plateEvent;
	}
}
