/*
	$Id: ModeInfo.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web;

import java.util.regex.Pattern;

import java.util.Map;
import java.util.HashMap;

import net.sf.basedb.core.Permission;

/**
	@author Nicklas
	@version 2.0
*/
public class ModeInfo
{

	private static final boolean HAS_TOOLBAR = true;
	private static final boolean NO_TOOLBAR = false;

	private static final boolean HAS_FILTER = true;
	private static final boolean NO_FILTER = false;

	private static final boolean HAS_NAVIGATOR = true;
	private static final boolean NO_NAVIGATOR = false;

	private static final boolean HAS_CHECK = true;
	private static final boolean NO_CHECK = false;

	private static final boolean HAS_RADIO = true;
	private static final boolean NO_RADIO = false;
	
	private static final boolean HAS_ICONS = true;
	private static final boolean NO_ICONS = false;

	private static final boolean IS_COMPACT = true;
	private static final boolean NOT_COMPACT = false;

	private static final boolean HAS_OKBUTTON = true;
	private static final boolean NO_OKBUTTON = false;

	private static final boolean HAS_CLOSEBUTTON = true;
	private static final boolean NO_CLOSEBUTTON = false;

	private static final boolean HAS_CANCELBUTTON = true;
	private static final boolean NO_CANCELBUTTON = false;
	
	private static final boolean HAS_PROPERTYLINK = true;
	private static final boolean NO_PROPERTYLINK = false;
	
	private static final boolean HAS_EDITLINK = true;
	private static final boolean NO_EDITLINK = false;

	private static final boolean SELECTION_MODE = true;
	private static final boolean NOT_SELECTION_MODE = false;
	
	private static final Permission NO_PERMISSION = null;
	
	private static final Map<String, ModeInfo> modes = new HashMap<String, ModeInfo>();

	public static final ModeInfo DEFAULT = 
		new ModeInfo(
			"default", "default", null,
			HAS_TOOLBAR, HAS_FILTER, HAS_NAVIGATOR, HAS_PROPERTYLINK,
			HAS_CHECK, NO_RADIO, HAS_ICONS, NOT_COMPACT, HAS_EDITLINK, 
			NO_OKBUTTON, NO_CLOSEBUTTON, NO_CANCELBUTTON, NO_PERMISSION, NOT_SELECTION_MODE
		);

	public static final ModeInfo SELECTONE = 
		new ModeInfo(
			"selectone", "popup", "Select one {one}",
			HAS_TOOLBAR, HAS_FILTER, HAS_NAVIGATOR, NO_PROPERTYLINK,
			NO_CHECK, HAS_RADIO, HAS_ICONS, IS_COMPACT, NO_EDITLINK,
			HAS_OKBUTTON, NO_CLOSEBUTTON, HAS_CANCELBUTTON, Permission.USE, SELECTION_MODE
		);

	public static final ModeInfo SELECTMULTIPLE = 
		new ModeInfo(
			"selectmultiple", "popup", "Select one or more {many}",
			HAS_TOOLBAR, HAS_FILTER, HAS_NAVIGATOR, NO_PROPERTYLINK,
			 HAS_CHECK, NO_RADIO, HAS_ICONS, IS_COMPACT, NO_EDITLINK, 
			HAS_OKBUTTON, NO_CLOSEBUTTON, HAS_CANCELBUTTON, Permission.USE, SELECTION_MODE
		);

	public static final ModeInfo SELECTMULTIPLE_NOBUTTONS =
		new ModeInfo(
				"selectmultiplenobuttons", "popup", "Select one or more {many}",
				HAS_TOOLBAR, HAS_FILTER, HAS_NAVIGATOR, NO_PROPERTYLINK,
				HAS_CHECK, NO_RADIO, HAS_ICONS, IS_COMPACT, NO_EDITLINK, 
				NO_OKBUTTON, NO_CLOSEBUTTON, NO_CANCELBUTTON, Permission.USE, SELECTION_MODE
			);
	
	private static final Pattern findOne = Pattern.compile("\\{one\\}");
	private static final Pattern findMany = Pattern.compile("\\{many\\}");
		
	public static ModeInfo get(String name)
	{
		ModeInfo mode = modes.get(name);
		return mode == null ? DEFAULT : mode;
	}

	private final String name;
	private final String pageType;
	private final String titleTemplate;
	private final boolean hasToolbar;
	private final boolean hasFilter;
	private final boolean hasNavigator;
	private final boolean hasCheck;
	private final boolean hasRadio;
	private final boolean hasIcons;
	private final boolean isCompact;
	private final boolean hasOkButton;
	private final boolean hasCloseButton;
	private final boolean hasCancelButton;
	private final boolean hasPropertyLink;
	private final boolean hasEditLink;
	private final Permission permission;
	private final boolean selectionMode;
	
	private ModeInfo(String name, String pageType, String titleTemplate,
		boolean hasToolbar, boolean hasFilter, boolean hasNavigator, boolean hasPropertyLink,
		boolean hasCheck, boolean hasRadio, boolean hasIcons, boolean isCompact, boolean hasEditLink,
		boolean hasOkButton, boolean hasCloseButton, boolean hasCancelButton, Permission permission, boolean selectionMode)
	{
		this.name = name;
		this.pageType = pageType;
		this.titleTemplate = titleTemplate;
		this.hasToolbar = hasToolbar;
		this.hasFilter = hasFilter;
		this.hasNavigator = hasNavigator;
		this.hasCheck = hasCheck;
		this.hasRadio = hasRadio;
		this.hasIcons = hasIcons;
		this.isCompact = isCompact;
		this.hasOkButton = hasOkButton;
		this.hasCloseButton = hasCloseButton;
		this.hasCancelButton = hasCancelButton;
		this.hasPropertyLink = hasPropertyLink;
		this.hasEditLink = hasEditLink;
		this.permission = permission;
		this.selectionMode = selectionMode;
		modes.put(name, this);
	}
	
	public String getName()
	{
		return name;
	}
	
	public String getPageType()
	{
		return pageType;
	}
	
	public boolean hasToolbar()
	{
		return hasToolbar;
	}
	
	public boolean hasFilter()
	{
		return hasFilter;
	}
	
	public boolean hasNavigator()
	{
		return hasNavigator;
	}

	public boolean hasCheck()
	{
		return hasCheck;
	}

	public boolean hasRadio()
	{
		return hasRadio;
	}

	public boolean hasIcons()
	{
		return hasIcons;
	}
	
	public boolean isCompact()
	{
		return isCompact;
	}

	public boolean hasOkButton()
	{
		return hasOkButton;
	}

	public boolean hasCloseButton()
	{
		return hasCloseButton;
	}

	public boolean hasCancelButton()
	{
		return hasCancelButton;
	}
	
	public boolean hasPropertyLink()
	{
		return hasPropertyLink;
	}
	
	public boolean hasEditLink()
	{
		return hasEditLink;
	}
	
	public Permission forceItemPermission()
	{
		return permission;
	}
	
	public boolean isSelectionMode()
	{
		return selectionMode;
	}
	
	public String generateTitle(String one, String many)
	{
		String result = titleTemplate;
		if (result != null)
		{
			result = findOne.matcher(result).replaceAll(one);
			result = findMany.matcher(result).replaceAll(many);
		}
		return result;
	}
	
}

