/**
	$Id: ServicesEventHandler.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.extensions.service;

import net.sf.basedb.clients.web.extensions.ExtensionsControl;
import net.sf.basedb.clients.web.extensions.Settings;
import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.extensions.ExtensionPoint;
import net.sf.basedb.util.extensions.events.EventHandler;
import net.sf.basedb.util.extensions.events.EventType;

/**
	Event handler for managing services when new extension are added
	or existing extension are updated or deleted.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class ServicesEventHandler
	implements EventHandler
{

	private static final org.apache.log4j.Logger log = 
		org.apache.log4j.LogManager.getLogger("net.sf.basedb.clients.web.extensions.service.ServicesEventHandler");

	private Settings settings;
	
	public ServicesEventHandler(Settings settings)
	{
		this.settings = settings;
	}
	
	@Override
	public void handleEvent(EventType event, ExtensionPoint<?> extensionPoint, Extension<?> extension)
	{
		if (log.isInfoEnabled())
		{
			log.info("Received event: " + event + "; extension point: " + extensionPoint + 
					"; extension: " + extension);
		}
		if (event == EventType.AFTER_REGISTRATION)
		{
			// Start services
			if (extension == null || settings.isEnabled(extension))
			{
				Services.start(extension);
			}
		}
		else if (event == EventType.BEFORE_UPDATE)
		{
			// Stop running services from this extension
			Services.stop(extension);
		}
		else if (event == EventType.AFTER_UPDATE)
		{
			// Re-start services that were running
			if (extension == null || settings.isEnabled(extension))
			{
				Services.start(extension);
			}
		}
		else if (event == EventType.BEFORE_UNREGISTRATION)
		{
			// Stop running services
			Services.stop(extension);
		}
		else if (event == ExtensionsControl.BEFORE_DISABLE)
		{
			// Stop running services
			Services.stop(extension);
		}
		else if (event == ExtensionsControl.AFTER_ENABLE)
		{
			// Start services
			Services.start(extension);
		}
		else
		{
			log.info("No handler for event " + event);
		}
	}

}
