/**
	$Id: ItemQueryWrapper.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.query.ResultIterator;

/**
	A query wrapper for {@link ItemQuery}:s.
	
	@author nicklas
	@version 2.2
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
public class ItemQueryWrapper
	implements QueryWrapper<BasicItem>
{
	private final ItemQuery<BasicItem> query;
	private final DbControl dc;
	private final Item itemType;
	private final Metadata metadata;
	
	/**
		Create a new query wrapper.
		@param dc The DbControl used to execute the query
		@param query The query
		@param itemType Type of the items returned by the query
	*/
	public ItemQueryWrapper(DbControl dc, ItemQuery<BasicItem> query, Item itemType)
	{
		this.dc = dc;
		this.query = query;
		this.itemType = itemType;
		this.metadata = Metadata.getInstance(itemType.getDataClass());
	}
	
	/**
		Calls {@link ItemQuery#iterate(DbControl)}
	*/
	public ResultIterator<BasicItem> iterate()
	{
		return query.iterate(dc);
	}
	
	/**
		Calls {@link BasicItem#getId()}
	*/
	public int getId(BasicItem item)
	{
		return item.getId();
	}
	
	/**
		Uses a {@link Metadata} object to get the value of the specified property.
	*/
	@SuppressWarnings("unchecked")
	public Object getData(ExportedProperty exportedProperty, BasicItem item)
	{
		return metadata.getPropertyValue(dc, item, exportedProperty.name);
	}
	
	
}
