/**
	$Id: QueryWrapper.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.plugins;

import net.sf.basedb.core.query.ResultIterator;

/**
	A helper interface for the {@link SimpleExport} class to make it
	easier to interact with different types of queries.

	@author nicklas
	@version 2.2
	@base.modified $Date: 2008-09-05 17:15:54 +0200 (Fri, 05 Sep 2008) $
*/
interface QueryWrapper<T>
{
	/**
		Get the {@link ResultIterator} for the query.
	*/
	public ResultIterator<T> iterate();
	
	/**
		Get the ID of the specified item. If the item doesn't have an ID a
		running number may be used. The important thing is that ID:s must be
		unique for each export.
		@param item The item
	*/
	public int getId(T item);
	
	/**
		Get the value for the specified property.
		@param exportedProperty The property to get the value for
		@param item The item that holds the value
	 * @throws Exception If failing to get the data.
	*/
	public Object getData(ExportedProperty exportedProperty, T item)
		throws Exception;
	
}
