/*
	$Id: Select.java 4478 2008-09-05 15:15:54Z jari $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.clients.web.taglib;

import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.Removable;

/**
	This tag is used to display a selection list of items. The list
	can be linked to a selection popup and has options for the current
	item and recently used items.
	<p>
	<b>Example:</b>
	<pre>
   &lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
   &lt;base:select 
   	id="protocol"
   	required="false"
   	current="&lt;%=currentProtocol%&gt;"
   	denied="false"
   	recent="&lt;%=recentProtocols%&gt;"
   	onselect="selectProtocolOnClick()"
   /&gt;
 </pre>

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:select
      id=...
      clazz=...
      style=...
      required=true|false
      current=...
      denied=true|false
      recent=...
      selectrecent=true|false
      defaultitem=...
      selectdefault=true|false
      onselect=...
      onchange=...
      newitem=true|false
      visible=true|false
      disabled=true|false
   &gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>yes</td>
		<td>The ID of the selection list. The value of this attribute
		goes directly into the standard <code>name</code> attribute of a &lt;select&gt;
		tag.
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td></td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different selection lists.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different selection lists.
		</td>
	</tr>
	<tr>
		<td>required</td>
		<td>false</td>
		<td>no</td>
		<td>
			If a value must be selected or not. If FALSE is specified
			the option <code>- none -</code> will be included.
		</td>
	</tr>
	<tr>
		<td>current</td>
		<td>null</td>
		<td>yes</td>
		<td>The current item that is selected. It should be a {@link BasicItem}.</td>
	</tr>
	<tr>
		<td>denied</td>
		<td>false</td>
		<td>no</td>
		<td>
			If null is passed as the current item, this parameter should be TRUE
			if the reason is that the logged in user is denied read access, FALSE
			otherwise.
		</td>
	</tr>
	<tr>
		<td>recent</td>
		<td></td>
		<td>no</td>
		<td>
			A list of recently used items. The should be of the same type as
			the current item.
		</td>
	</tr>
	<tr>
		<td>selectrecent</td>
		<td>true</td>
		<td>no</td>
		<td>
			If no current has been specified and it is a new item then, if this
			parameter is TRUE the first recently used value is selected. If this
			parameter is FALSE, no item is selected.
		</td>
	</tr>
	<tr>
		<td>defaultitem</td>
		<td></td>
		<td>no</td>
		<td>
			Default value set e.g. for a project.
		</td>
	</tr>
	<tr>
		<td>selectdefault</td>
		<td>true</td>
		<td>no</td>
		<td>
			If no current or recent has been specified and it is a new item then, if this
			parameter is TRUE the default value is selected. If this
			parameter is FALSE, no item is selected.
		</td>
	</tr>
	<tr>
		<td>onchange</td>
		<td></td>
		<td>no</td>
		<td>
			A javascript statement that is executed when the another item
			in the list is selected.
		</td>
	</tr>
	<tr>
		<td>onselect</td>
		<td></td>
		<td>no</td>
		<td>
			If specified, a <code>Select</code> button is added and
			linked to this javascript call.
		</td>
	</tr>
	<tr>
		<td>newitem</td>
		<td>false</td>
		<td>no</td>
		<td>
			If the parent item is a new item or an already saved item. This
			affects the ID of the current item that is used in the list. For
			existing values the ID is negative, which means that the link
			doesn't have to change unless another item is selected.
		</td>
	</tr>
	<tr>
		<td>visible</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the selection list should be visible or not.
		</td>
	</tr>
	<tr>
		<td>disabled</td>
		<td>false</td>
		<td>no</td>
		<td>
			If the selection list should be enabled or not.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
*/
public class Select
	extends TagSupport
{

	
	/**
		
	*/
	private static final long serialVersionUID = -9096279950931870965L;

	/**
		Optional <code>class</code> attribute
	*/
	private String clazz = null;
	
	/**
		Optional <code>style</code> attribute
	*/
	private String style = null;

	/**
		If a value is required or not.
	*/
	private boolean required;
	
	/**
		The currently selected item.
	*/
	private BasicItem current;
	
	/**
		If the logged in user was denied access to the current item.
	*/
	private boolean denied = false;
	
	/**
		List of recently used items.
	*/
	private List<? extends BasicItem> recent;
	
	/**
		If we can select the most recently used item for new items
		with nu current item.
	*/
	private boolean selectRecent = true;
	
	/**
	  	The project default value
	*/
	private BasicItem defaultitem;
	
	/**
	 	If the default item should be selected for new items with
	 	no current or recently used items.
	*/
	private boolean selectdefault = true;
	
	/**
		JavaScript statement.
	*/
	private String onChange = null;
	
	/**
		JavaScript statement.
	*/
	private String onSelect = null;
	
	/**
		If the link is from a new item or an old item.
	*/
	private boolean newItem = false;
	
	/**
		If the select list should be visible or not.
	*/
	private boolean visible = true;

	/**
		If the selection list should be enabled or not.
	*/
	private boolean disabled = false;


	/*
		Taglib initialization methods
	*/
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}
	public String getClazz()
	{
		return clazz;
	}

	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}
	
	public void setRequired(boolean required)
	{
		this.required = required;
	}
	public boolean isRequired()
	{
		return required;
	}

	public void setCurrent(BasicItem current)
	{
		this.current = current;
	}
	public BasicItem getCurrent()
	{
		return current;
	}
	
	public void setDenied(boolean denied)
	{
		this.denied = denied;
	}
	public boolean isDenied()
	{
		return denied;
	}

	public void setRecent(List<? extends BasicItem> recent)
	{
		this.recent = recent;
	}
	public List<? extends BasicItem> getRecent()
	{
		return recent;
	}
	
	public void setSelectrecent(boolean selectRecent)
	{
		this.selectRecent = selectRecent;
	}
	public boolean getSelectrecent()
	{
		return selectRecent;
	}

	public void setDefaultitem(BasicItem defaultitem)
	{
		this.defaultitem = defaultitem;
	}
	public BasicItem getDefaultitem()
	{
		return defaultitem;
	}
	
	public void setSelectdefault(boolean selectdefault)
	{
		this.selectdefault = selectdefault;
	}
	public boolean getSelectdefault()
	{
		return selectdefault;
	}
	
	public void setOnchange(String onChange)
	{
		this.onChange = onChange;
	}
	public String getOnchange()
	{
		return onChange;
	}
	
	public void setOnselect(String onSelect)
	{
		this.onSelect = onSelect;
	}
	public String getOnselect()
	{
		return onSelect;
	}

	public void setNewitem(boolean newItem)
	{
		this.newItem = newItem;
	}
	public boolean isNewitem()
	{
		return newItem;
	}
	
	public void setVisible(boolean visible)
	{
		this.visible = visible;
	}
	public boolean isVisible()
	{
		return visible;
	}

	public void setDisabled(boolean disabled)
	{
		this.disabled = disabled;
	}
	public boolean isDisabled()
	{
		return disabled;
	}

	public int doStartTag()
		throws JspException
	{
		if (!isVisible()) return SKIP_BODY;
//		Page page = (Page)findAncestorWithClass(this, Page.class);

		
		StringBuilder sb = new StringBuilder();

		sb.append("<div");
		if (getId() != null) sb.append(" id=\"").append(getId()).append("\"");
		if (getClazz() != null) sb.append(" class=\"").append(getClazz()).append("\"");
		if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
		sb.append(">\n");
		sb.append("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n");
		sb.append("<tr><td>");
		boolean disabled = isDenied() || isDisabled();
		sb.append("<select name=\"").append(getId()).append("\"");
		sb.append(disabled ? " disabled readonly class=\"disabled\"" : isRequired() ? " class=\"required\"" : "");
		if (getOnchange() != null) sb.append(" onchange=\"").append(getOnchange()).append("\"");
		sb.append(">\n");
		if (isDenied())
		{
			sb.append("<option value=\"-1\">- denied -\n");
		}
		else
		{
			if (!isRequired())
			{
				sb.append("<option value=\"0\">- none -\n");
			}
			if (current != null)
			{
				sb.append("<option value=\"");
				sb.append(current.getId() * (isNewitem() ? 1 : -1));
				sb.append("\" selected>");
				String name = getName(current);
				sb.append(HTML.encodeTags(name)).append("\n");
			}
			if (recent != null && recent.size() > 0)
			{
				sb.append("<option value=\"0\" disabled=\"true\" class=\"recentheader\">- recently used -\n");
				int i = 1;
				for (BasicItem item : recent)
				{
					String name = getName(item);
					String clazz = "recent";
					if (item instanceof Removable && ((Removable)item).isRemoved())
					{
						clazz += " removed";
						name += " (in trashcan)";
					}
					sb.append("<option value=\"").append(item.getId()).append("\"");
					sb.append(" class=\"").append(clazz).append("\"");
					if (i == 1 && current == null && isNewitem() && getSelectrecent()) 
					{
						sb.append(" selected");
					}
					sb.append(">").append(i).append(". ").append(name).append("\n");
					i++;
				}
			}
			if (defaultitem != null)
			{
				sb.append("<option value=\"0\" disabled=\"true\" class=\"defaultheader\">- default -\n");
				String name = getName(defaultitem);
				String clazz = "default";
				
				sb.append("<option value=\"").append(defaultitem.getId()).append("\"");
				sb.append(" class=\"").append(clazz).append("\"");				
				if ( current == null && 
						(recent != null && recent.isEmpty()) && 
						isNewitem() && 
						getSelectdefault() )
				{					
					sb.append(" selected");
				}
				sb.append(">").append(name).append("\n");
			}
		}
		sb.append("</select>&nbsp;\n");
		sb.append("</td>");
		if (getOnselect() != null)
		{
			sb.append("<td>");
			sb.append("<div class=\"buttonclass").append(disabled ? "_disabled": "").append("\"");
			if (!disabled)
			{
				sb.append(" onmouseover=\"this.className='buttonclass_hover';\" onmouseout=\"this.className='buttonclass';\"");
				sb.append(" onclick=\"").append(getOnselect()).append("\"");
			}
			sb.append("><div class=\"buttonclass_inner\">Select&hellip;</div></div>");
			sb.append("</td>\n");
		}
		sb.append("</tr></table></div>");

		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}

	private String getName(BasicItem item)
	{
		String name = null;
		if (item instanceof Nameable)
		{
			name = ((Nameable)item).getName();
		}
		else
		{
			name = item.toString();
		}
		return name;
	}
	
}




