/*
	$Id: DynamicExtraValueQuery.java 4479 2008-09-05 15:16:27Z jari $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Select;

/**
	Represents a query for extra values in the dynamic database. A single
	query object may query against data in one bioassayset at a time.
	<p>
	This type of query is executed by the {@link #iterate(DbControl)} method
	and returns the result as a {@link DynamicResultIterator}.
	
	@author Nicklas
	@version 2.0
	@see ExtraValue#getValues()
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class DynamicExtraValueQuery
	extends DynamicQuery
{

	private final Select[] defaultSelects;

	/**
		Create a new dynamic query.
		@param cube The cube to query against
		@param rootTable The root table of the query
		@param defaultSelects The columns in the root table that are
			selected by default if no other selections are specfied before
			the query is executed
	*/
	DynamicExtraValueQuery(DataCube cube, VirtualTable rootTable, VirtualColumn... defaultSelects)
	{
		super(cube, rootTable);
		this.defaultSelects = new Select[defaultSelects.length];
		for (int i = 0; i < defaultSelects.length; ++i)
		{
			this.defaultSelects[i] = Dynamic.select(defaultSelects[i]);
		}
	}
	
	/*
		From the AbstractSqlQuery class
		-------------------------------------------
	*/
	/**
		Default selection: column, position, extraValue
	*/
	Select[] getDefaultSelects()
	{
		return defaultSelects;
	}
	// -------------------------------------------

}
