/*
	$Id: Identifiable.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

/**
	An <code>Identifiable</code> item is an item which has an
	<code>id</code> a <code>type</code> and a <code>version</code>.
	The id value is always greater than 0 for an existing item, and 0 for
	a new item that has not yet been saved to the database.
	<p>
	The type is an value selected from the constants
	defined by the {@link Item} enumeration.

	@author Nicklas
	@version 2.0
	@see BasicItem
	@see Item
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface Identifiable
	extends AccessControlled
{

	/**
		Get the id for the item. If it hasn't yet been saved to the
		database 0 is returned.
		@return The id of the item or 0
	*/
	public int getId();

	/**
		Get the type of item represented by the object. The returned
		value is one of the values defined in the {@link Item} enumeration.
		@return A value indicating the type of item
	*/
	public Item getType();
	
	/**
		Get the version number of the item. The version number is incremented each time
		the item is updated and is used to prevent that the same item is updated at
		the same time by two or more different threads or processes.
	*/
	public int getVersion();

}
