/*
	$Id: MeanRawDataPropertyExpression.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.query.Aggregations;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Query;
import net.sf.basedb.core.query.QueryElement;
import net.sf.basedb.core.query.QueryType;
import net.sf.basedb.core.query.Expression;
import net.sf.basedb.core.query.Selects;

import java.sql.SQLException;
import java.util.Collection;

/**
	An expression representing the mean value of property of a
	raw data spot.
	<p>
	Throws an <code>UnsupportedOperationException</code> if 
	{@link Query#getQueryType()} isn't {@link QueryType#SQL}.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.query.Dynamic#meanRawData(String)
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class MeanRawDataPropertyExpression
	implements Expression
{
	private final String property;
	
	public MeanRawDataPropertyExpression(String property)
		throws InvalidDataException
	{
		this.property = property;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL)
		{
			// TODO - this is not very elegant
			RawBioAssay rawBioAssay = null;
			if (query instanceof DynamicRawDataQuery)
			{
				 rawBioAssay = ((DynamicRawDataQuery)query).getRawBioAssay();
			}
			if (rawBioAssay == null) throw new InvalidDataException("Could not find raw bioassay for query: " +query);

			DynamicRawDataQuery meanQuery = rawBioAssay.getDynamicRawData();
			meanQuery.select(Selects.expression(Aggregations.mean(Dynamic.rawData(property)), "m"));
			try
			{
				return Float.toString(meanQuery.iterate(dc).next().getFloat(1));
			}
			catch (SQLException ex)
			{
				throw new BaseException(ex);
			}
		}
		else
		{
			throw new UnsupportedOperationException("'meanRawDataProperty' expression not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "AVG(raw." + property+")";
	}
	// -------------------------------------------
	
}
