/*
	$Id: PathParameterType.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.
	
	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ParameterValueData;
import net.sf.basedb.core.data.TextParameterValueData;

import java.util.List;

/**
	This class represent a parameter type that is a path to either a
	directory or a file. This parameter type should be used when a plugin
	needs to ask for a directory or file to save it's output.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class PathParameterType
	extends ParameterType<String>
{
	private final Path.Type pathType;

	/**
		Create a new optional path parameter type.
		@param pathType If the path is to a directory or file
	*/
	public PathParameterType(Path.Type pathType)
	{
		this(pathType, null, false);
	}

	/**
		Create a new path parameter type.
		@param pathType If the path is to a directory or file
		@param defaultValue The default value of this parameter
		@param notNull FALSE if nulls values are allowed, TRUE otherwise
	*/
	public PathParameterType(Path.Type pathType, String defaultValue, boolean notNull)
	{
		super(String.class, defaultValue, notNull);
		this.pathType = pathType;
	}
	
	public PathParameterType(Path.Type pathType, String defaultValue, boolean notNull, int multiplicity, int width, int height, List<String> items)
	{
		super(String.class, defaultValue, notNull, multiplicity, width, height, items);
		this.pathType = pathType;
	}

	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "PathParameterType: type " + pathType;
	}
	// -------------------------------------------

	/*
		From the ParameterType class
		-------------------------------------------
	*/
	/**
		Checks if the value is a valif {@link Path} value.
		@param value The value to test
		@throws InvalidDataException If the value is not a valid path
	*/
	void validateValue(String name, String value)
		throws InvalidDataException
	{
		try
		{
			new Path(value, pathType);
		}
		catch (InvalidPathException ex)
		{
			throw new InvalidPathException("Path for parameter '"+ name + "' is invalid: " + ex.getMessage());
		}
	}
	/**
		Create a new {@link TextParameterValueData} object.
	*/
	@Override
	ParameterValueData<String> newParameterValueData()
	{
		return new TextParameterValueData();
	}
	// -------------------------------------------

	/**
		Get the type of the path to ask for.
		@return A <code>Path.Type</code> object
	*/
	public Path.Type getPathType()
	{
		return pathType;
	}
}
