/*
	$Id: PermissionDeniedException.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005, 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

/**
	This exception is thrown when trying to load or manipulate an 
	item in the database and you don't have the required permission.

	@author Nicklas
	@version 2.0
*/
public class PermissionDeniedException
	extends BaseException
{
	/**
		
	*/
	private static final long serialVersionUID = 736019555576108955L;
	/**
		Creates a new <code>PermissionDeniedException</code>. The error 
		message produced will look like: <code>Permission denied: Not 
		allowed to write</code>

		@param permission The permission requested. Should be one of the 
			values defined in the {@link Permission} class}
	*/
	public PermissionDeniedException(Permission permission)
	{
		super("Permission denied: Not allowed to "+
			(permission == Permission.DENIED ? "access" : permission.toString()));
	}

	/**
		Creates a new <code>PermissionDeniedException</code>. The error 
		message produced will look like: <code>Permission denied: 
		Not allowed to read User[ID=1]</code>

		@param permission The permission requested. Should be one of the 
			values defined in the {@link Permission} class}
		@param what A description of what access is denied to, for
			example: User[ID=1]
	*/
	public PermissionDeniedException(Permission permission, String what)
	{
		super("Permission denied: Not allowed to "+
			(permission == Permission.DENIED ? "access" : permission.toString())+" "+what);
	}

	/**
		Creates a new <code>PermissionDeniedException</code>. The error
		message produced will look like: 
		<code>Permission denied: <i>message</i></code>

		@param message The message explaining why permission was denied
	*/
	public PermissionDeniedException(String message)
	{
		super("Permission denied: "+message);
	}
	/**
		Creates a new <code>PermissionDeniedException</code>. The error
		message produced will look like: 
		<code>Permission denied: <i>message</i></code>
	
		@param message The message explaining why permission was denied
		@param cause The original cause of the exception
	*/
	public PermissionDeniedException(String message, Throwable cause)
	{
		super("Permission denied: "+message, cause);
	}

}



