/*
  $Id: RealTable.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

/**
	This class defines constants that are used to specify a table in
	the regular (non-dynamic) database while doing analysis with dynamic
	queries. This makes it possible to join data from the regular tables 
	into a dynamic query.

	@author Nicklas
	@version 2.0
	@see DynamicSpotQuery#joinReporters(net.sf.basedb.core.query.JoinType)
	@see net.sf.basedb.core.RealJoin
	@see RawDataType#getRealTable()
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class RealTable
{

	public static final RealTable REPORTERS = new RealTable("Reporters", "rpt");
	
	public static final RealTable REPORTER_LIST_SCORES = new RealTable("ReporterListScores", "rls");

	/**
		The unquoted table name.
	*/
	private final String tableName;
	/**
		The alias to use in queries for this table.
	*/
	private final String alias;
	
	/**
	 	Creates a new table.
		@param tableName The name of the table
		@param alias The alias of the table used in queries
	*/
	RealTable(String tableName, String alias)
	{
		this.tableName = tableName;
		this.alias = alias;
	}
	
	/**
		Get the unquoted table name of this table in the database.
	*/
	public String getTableName()
	{
		return tableName;
	}
	
	/**
		Get the quoted table name of this table in the database.
	*/
	public String getQualifiedTableName()
	{
		return HibernateUtil.quote(tableName);
	}
	
	/**
		Get the alias of this table. Used in queries:
		<code>SELECT ... FROM table AS alias ... </code>
	*/
	public String getAlias()
	{
		return alias;
	}
	
	/**
		Get the alias of this table, using the ID of the item to make it 
		unique. Used in queries:
		<code>SELECT ... FROM table AS alias ... </code>
	*/
	public String getAlias(BasicItem item)
	{
		return alias + item.getId();
	}
	
	/**
		Get the alias of this table, using the ID of the item to make it 
		unique. Used in queries:
		<code>SELECT ... FROM table AS alias ... </code>
		@since 2.8
	*/
	public String getAlias(int id)
	{
		return alias + id;
	}
	

}
