/*
	$Id: SpotImage.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core;

import java.io.InputStream;
import java.io.ByteArrayInputStream;

/**
	Holds the image information about a single spot image
	in PNG format.

	@author Samuel, Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class SpotImage
{
	private final byte[] image;

	/**
		Constructor that takes the image as a byte array.
		@param image Byte array of a PNG image
	*/
	SpotImage(byte[] image)
	{
		this.image = image;
	}


	/**
		Get an <code>InputStream</code> of the image.
		@return An <code>InputStream</code>
	*/
	public InputStream getInputStream()
	{
		return  new ByteArrayInputStream(image);
	}

	/**
		Get the image as an array of bytes. The returned array is a copy of the 
		original image.
		@return An array of bytes representing the image in PNG format
	*/
	public byte[] getImage()
	{
		byte[] copy = new byte[image.length];
		System.arraycopy(image, 0, copy, 0, image.length);
		return copy;
	}
	
	/**
		Get the number of bytes of the image.
	*/
	public int getBytes()
	{
		return image.length;
	}
}
