/**
	$Id: AnyToAnyData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

/**
 	Represents a link from one item to any other item. The links are loosely
 	coupled and are not using foreign keys.

	@author nicklas
	@version 2.0
	@see net.sf.basedb.core.AnyToAny
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see <a href="../../../../../../html/developerdoc/api/data_api.misc.html">Misc overview</a>
	@hibernate.class table="`AnyToAny`" lazy="false"
*/
public class AnyToAnyData
	extends BasicData
	implements NameableData
{

	public AnyToAnyData()
	{}
	
	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	/**
		@hibernate.property type="string"
		@hibernate.column name="`name`" length="255" not-null="true" index="name_idx" unique-key="uniquelink"
	*/
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	
	private int fromId;
	/**
		The ID of the item we are linking from.
		@hibernate.property type="int"
		@hibernate.column name="`from_id`" not-null="true" unique-key="uniquelink"
	*/
	public int getFromId()
	{
		return fromId;
	}
	public void setFromId(int fromId)
	{
		this.fromId = fromId;
	}

	private int fromType;
	/**
		The type code of the item we are linking from.
		@hibernate.property type="int"
		@hibernate.column name="`from_type`" not-null="true" unique-key="uniquelink"
	*/
	public int getFromType()
	{
		return fromType;
	}
	public void setFromType(int fromType)
	{
		this.fromType = fromType;
	}
	
	private int toId;
	/**
		The ID of the item we are linking to.
		@hibernate.property type="int"
		@hibernate.column name="`to_id`" not-null="true" index="toitem"
	*/
	public int getToId()
	{
		return toId;
	}
	public void setToId(int toId)
	{
		this.toId = toId;
	}

	private int toType;
	/**
		The type code of the item we are linking to.
		@hibernate.property type="int" 
		@hibernate.column name="`to_type`" not-null="true" index="toitem"
	*/
	public int getToType()
	{
		return toType;
	}
	public void setToType(int toType)
	{
		this.toType = toType;
	}
	
	private boolean usesTo;
	/**
		If the link counts as using the item we are linking to.
		@hibernate.property column="`uses_to`" type="boolean" not-null="true"
	*/
	public boolean isUsingTo()
	{
		return usesTo;
	}
	public void setUsingTo(boolean usesTo)
	{
		this.usesTo = usesTo;
	}

}
