/*
 $Id: ArrayDesignData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 */
package net.sf.basedb.core.data;

import java.util.HashSet;
import java.util.Set;

/**
	This class holds information about an arrayDesign
	
	@author enell
	@version 2.0
	@see net.sf.basedb.core.ArrayDesign
	@see <a href="../../../../../../html/developerdoc/api/data_api.arrays.html">Array LIMS - arrays overview</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@hibernate.class table="`ArrayDesigns`" lazy="true"
*/
public class ArrayDesignData 
	extends AnnotatedData
	implements FileStoreEnabledData
{
	public ArrayDesignData()
	{}

	/*
		From the FileStoreEnabledData interface
		-------------------------------------------
	*/
	private FileSetData fileSet;
	public FileSetData getFileSet()
	{
	   return fileSet;
	}
	public void setFileSet(FileSetData fileSet)
	{
	   this.fileSet = fileSet;
	}
	// -------------------------------------------
	
	private PlatformData platform;
	/**
		Get the platform this raw bioassay uses
		@since 2.5
		@hibernate.many-to-one outer-join="false"
		@hibernate.column name="`platform_id`" not-null="true"
	*/
	public PlatformData getPlatform()
	{
		return platform;
	}
	public void setPlatform(PlatformData platform)
	{
		this.platform = platform;
	}
	
	private PlatformVariantData variant;
	/**
		Get the platform variant this raw bioassay uses, or null.
		@since 2.5
		@hibernate.many-to-one outer-join="false"
		@hibernate.column name="`variant_id`"
	*/
	public PlatformVariantData getVariant()
	{
		return variant;
	}
	public void setVariant(PlatformVariantData variant)
	{
		this.variant = variant;
	}
	
	private JobData job;
	/**
		Get the job that was used to import features to this array desing
		@hibernate.many-to-one column="`job_id`" not-null="false" outer-join="false"
		@since 2.7
	*/
	public JobData getJob()
	{
		return job;
	}
	/**
		@since 2.7
	*/
	public void setJob(JobData job)
	{
		this.job = job;
	}
	
	private int numArrays;
	/**
		The number of arrays on a single slide.
		@hibernate.property column="`num_arrays`" type="int" not-null="true"
		@since 2.6
	*/
	public int getNumArrays()
	{
		return numArrays;
	}
	public void setNumArrays(int numArrays)
	{
		this.numArrays = numArrays;
	}
	
	private boolean affyChip;
	/**
		Check if this design is an Affymetrix chip.
		@hibernate.property column="`affy_chip`" type="boolean" not-null="true" 
		@deprecated Has been replaced by platform
	*/
	public boolean isAffyChip()
	{
		return affyChip;
	}
	/**
		@deprecated Has been replaced by platform
	*/
	public void setAffyChip(boolean affyChip)
	{
		this.affyChip = affyChip;
	}

	private boolean hasFeatures;
	/**
		If the array design has features or not.
		@hibernate.property column="`has_features`" type="boolean" not-null="true"
	*/
	public boolean getHasFeatures()
	{
		return hasFeatures;
	}
	public void setHasFeatures(boolean hasFeatures)
	{
		this.hasFeatures = hasFeatures;
	}

	private int dbFeatures;
	/**
		The number of features on this array design that are stored in 
		the database.
		@hibernate.property column="`db_features`" type="int" not-null="true"
		@since 2.5
	*/
	public int getNumDbFeatures()
	{
		return dbFeatures;
	}
	public void setNumDbFeatures(int dbFeatures)
	{
		this.dbFeatures = dbFeatures;
	}
	
	private int fileFeatures;
	/**
		The number of features on this array design that are stored inside
		files.
		@hibernate.property column="`file_features`" type="int" not-null="true"
		@since 2.5
	*/
	public int getNumFileFeatures()
	{
		return fileFeatures;
	}
	public void setNumFileFeatures(int fileFeatures)
	{
		this.fileFeatures = fileFeatures;
	}
	
	private int featureIdentificationMethod;
	/**
		The method to use for identiying features.
		1 = COORDINATES, 2 = POSITION, 3 = FEATURE_ID
		@hibernate.property column="`identification_method`" type="int" not-null="true"
		@since 2.6
	*/
	public int getFeatureIdentificationMethod()
	{
		return featureIdentificationMethod;
	}
	public void setFeatureIdentificationMethod(int featureIdentificationMethod)
	{
		this.featureIdentificationMethod = featureIdentificationMethod;
	}
	
	
	private Set<ArrayDesignBlockData> arrayDesignBlocks;
	/**
		This is the inverse end.
		@see ArrayDesignBlockData#getArrayDesign()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`arraydesign_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ArrayDesignBlockData"
	*/
	Set<ArrayDesignBlockData> getArrayDesignBlocks()
	{
		return arrayDesignBlocks;
	}
	void setArrayDesignBlocks(Set<ArrayDesignBlockData> arrayDesignBlocks)
	{
		this.arrayDesignBlocks = arrayDesignBlocks;
	}

	private Set<ArrayDesignPlateData> plates;
	/**
		The plates this design is made from.
		
		@hibernate.set lazy="true" inverse="true" cascade="all-delete-orphan"
		@hibernate.collection-key column="`arraydesign_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ArrayDesignPlateData"
	*/
	public Set<ArrayDesignPlateData> getPlates()
	{
		if (plates == null)
		{
			plates = new HashSet<ArrayDesignPlateData>();
		}
		return plates;
	}
	void setPlates(Set<ArrayDesignPlateData> plates)
	{
		this.plates = plates;
	}
	
	private Set<ArrayBatchData> arrayBatches;
	/**
		This is the inverse end.
		@see ArrayBatchData#getArrayDesign()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`arraydesign_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ArrayBatchData"
	*/
	Set<ArrayBatchData> getArrayBatches()
	{
		return arrayBatches;
	}
	void setArrayBatches(Set<ArrayBatchData> arrayBatches)
	{
		this.arrayBatches = arrayBatches;
	}
	
}
