/*
  $Id: FileSetData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.util.HashMap;
import java.util.Map;

/**
	This class collects all files used to store data
	for an item that store data in files instead of in the
	database.
	
	@author Nicklas
	@version 2.5
	@see net.sf.basedb.core.FileSet
	@see <a href="../../../../../../html/developerdoc/api/data_api.platforms.html">Data API - Experimental platforms</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@hibernate.class table="`FileSets`" lazy="true"
*/
public class FileSetData
	extends BasicData
{

	private int itemType;
	/**
		Get the item type this file set is associated with.
		@hibernate.property column="`item_type`" type="int" not-null="true" update="false"
	*/
	public int getItemType()
	{
		return itemType;
	}
	public void setItemType(int itemType)
	{
		this.itemType = itemType;
	}
	
	private Map<DataFileTypeData, FileSetMemberData> members;
	/**
		The members of this file set. This is the inverse end.
		@see FileSetMemberData#getFileSet()
		@hibernate.map lazy="true" cascade="delete" inverse="true"
		@hibernate.collection-key column="`fileset_id`"
		@hibernate.index-many-to-many column="`datafiletype_id`" class="net.sf.basedb.core.data.DataFileTypeData"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.FileSetMemberData"
	*/
	public Map<DataFileTypeData, FileSetMemberData> getMembers()
	{
		if (members == null)
		{
			members = new HashMap<DataFileTypeData, FileSetMemberData>();
		}
		return members;
	}
	void setMembers(Map<DataFileTypeData, FileSetMemberData> members)
	{
		this.members = members;
	}

}
