/*
  $Id: FileStoreEnabledData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

/**
	Items implementing this interface can store data in files
	instead of in the database. Files are collected in file
	sets.
	<p>
	<p>
	This interface defines Hibernate database mapping for the
	<code>fileSet</code> property to the database column <code>fileset_id</code>.
	If a subclass wants to map the property to another column, 
	it should override the {@link #getFileSet()} method and add a 
	Hibernate tag in the comment.
	<p>
	
	<b>Reference implementation</b><br>
	<pre class="code">
private FileSetData fileSet;
public FileSetData getFileSet()
{
   return fileSet;
}
public void setFileSet(FileSetData fileSet)
{
   this.fileSet = fileSet;
}
</pre>

	@author Nicklas
	@version 2.5
	@see <a href="../../../../../../html/developerdoc/api/data_api.platforms.html">Data API - Experimental platforms</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface FileStoreEnabledData
	extends IdentifiableData
{
	/**
		Get the file set attached to this item.
		@return A {@link FileSetData} object or null if no files are attached
		@hibernate.many-to-one column="`fileset_id`" not-null="false" outer-join="false"
			unique="true" cascade="delete"
	*/
	public FileSetData getFileSet();

	/**
		Attach a file set to the item. Null is allowed to unlink it.
	*/
	public void setFileSet(FileSetData file);
}
