/*
	$Id: JobAgentData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.util.HashMap;
import java.util.Map;

/**
	This class holds information about a job agent.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.JobAgent
	@see <a href="../../../../../../html/developerdoc/api/data_api.plugins.html">Plugin/jobs overview</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@hibernate.class table="`JobAgents`" lazy="false"
*/
public class JobAgentData
	extends CommonData
{

	public JobAgentData()
	{}

	/**
		The maximum length of the external ID that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external ID for the job agent.
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false" unique="true"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}
	
	/**
		The maximum length of the server that can be stored in the database.
		@see #setServer(String)
	*/
	public static final int MAX_SERVER_LENGTH = 255;
	private String server;
	/**
		Get the server name or ip that this jobagent is running on.
		@hibernate.property column="`server`" type="string" length="255"
	*/
	public String getServer()
	{
		return server;
	}
	public void setServer(String server)
	{
		this.server = server;
	}
	
	private Integer port;
	/**
		The port the job agent is listening on. null if it doesn't listen.
		@hibernate.property column="`port`" type="int" not-null="false"
	*/
	public Integer getPort()
	{
		return port;
	}
	public void setPort(Integer port)
	{
		this.port = port;
	}
	
	private Map<PluginDefinitionData, JobAgentSettingsData> plugins;
	/**
		The plugins that can be executed by this job agent. This is the
		inverse end.
		@see JobAgentSettingsData#getJobAgent()
		@hibernate.map lazy="true" cascade="delete" inverse="true"
		@hibernate.collection-key column="`jobagent_id`"
		@hibernate.index-many-to-many column="`plugindefinition_id`" class="net.sf.basedb.core.data.PluginDefinitionData"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.JobAgentSettingsData"
	*/
	public Map<PluginDefinitionData, JobAgentSettingsData> getPlugins()
	{
		if (plugins == null)
		{
			plugins = new HashMap<PluginDefinitionData, JobAgentSettingsData>();
		}
		return plugins;
	}
	void setPlugins(Map<PluginDefinitionData, JobAgentSettingsData> plugins)
	{
		this.plugins = plugins;
	}

}
