/*
	$Id: JobData.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;
import java.util.Map;
import java.util.HashMap;

/**
	This class holds information about a job.

	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Job
	@see <a href="../../../../../../html/developerdoc/api/data_api.plugins.html">Plugin/jobs overview</a>
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@hibernate.class table="`Jobs`" lazy="false"
*/
public class JobData
	extends OwnedData
	implements NameableData, RemovableData
{

	public JobData()
	{}

	/*
		From the NameableData interface
		-------------------------------------------
	*/
	private String name;
	public String getName()
	{
		return name;
	}
	public void setName(String name)
	{
		this.name = name;
	}
	private String description;
	public String getDescription()
	{
		return description;
	}
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private boolean removed;
	public boolean isRemoved()
	{
		return removed;
	}
	public void setRemoved(boolean removed)
	{
		this.removed = removed;
	}
	// -------------------------------------------

	private int type;
	/**
		The type of job: 1 = run plugin, 2 = other
		@hibernate.property column="`type`" type="int" not-null="true"
	*/
	public int getType()
	{
		return type;
	}
	public void setType(int type)
	{
		this.type = type;
	}
	
	private PluginDefinitionData pluginDefinition;
	/**
		The plugin that executes the job.
		@hibernate.many-to-one column="`plugindefinition_id`" not-null="false" outer-join="false" update="false"
	*/
	public PluginDefinitionData getPluginDefinition()
	{
		return pluginDefinition;
	}
	public void setPluginDefinition(PluginDefinitionData pluginDefinition)
	{
		this.pluginDefinition = pluginDefinition;
	}

	
	private PluginConfigurationData pluginConfiguration;
	/**
		The plugin configuration for the plugin.
		@hibernate.many-to-one column="`pluginconfiguration_id`" not-null="false" outer-join="false" update="false"
	*/
	public PluginConfigurationData getPluginConfiguration()
	{
		return pluginConfiguration;
	}
	public void setPluginConfiguration(PluginConfigurationData pluginConfiguration)
	{
		this.pluginConfiguration = pluginConfiguration;
	}

	private ExperimentData experiment;
	
	/**
		The experiment this job is a part of.
		@since 2.4
		@hibernate.many-to-one column="`experiment_id`" not-null="false" outer-join="false"
	*/
	public ExperimentData getExperiment()
	{
		return experiment;
	}
	public void setExperiment(ExperimentData experiment)
	{
		this.experiment = experiment;
	}
	
	
	/**
		The maximum allowed length of the plugin version.
		@since 2.8
	*/
	public static final int MAX_PLUGIN_VERSION_LENGTH = PluginDefinitionData.MAX_VERSION_LENGTH;
	private String pluginVersion;
	/**
		Get the version of the plugin used to execute this job at the time the
		job was executed or registered.
		@return A string with the version of the plugin, or null if not known
			or if the job is not executed by a plug-in
		@hibernate.property column="`version_string`" type="string" length="255" not-null="false"
		@since 2.8
	*/
	public String getPluginVersion()
	{
		return pluginVersion;
	}
	/**
		@since 2.8
	*/
	public void setPluginVersion(String pluginVersion)
	{
		this.pluginVersion = pluginVersion;
	}
	
	private int parameterVersion;
	/**
		Get the parameter version of the configuration  used for this job.
		@hibernate.property column="`parameter_version`" type="int" not-null="true"
	*/
	public int getParameterVersion()
	{
		return parameterVersion;
	}
	public void setParameterVersion(int parameterVersion)
	{
		this.parameterVersion = parameterVersion;
	}
	
	private boolean sendMessage;
	/**
		If the core should send a message to the owner when the job is finished
		or not.
		@hibernate.property column="`send_message`" type="boolean" not-null="true"
	*/
	public boolean getSendMessage()
	{
		return sendMessage;
	}
	public void setSendMessage(boolean sendMessage)
	{
		this.sendMessage = sendMessage;
	}
	
	private boolean removeJobWhenFinished;
	/**
		If the core should delete the job when it's finished successfully.
		@hibernate.property column="`remove_job`" type="boolean" not-null="true"
	*/
	public boolean getRemoveJobWhenFinished()
	{
		return removeJobWhenFinished;
	}
	public void setRemoveJobWhenFinished(boolean removeJobWhenFinished)
	{
		this.removeJobWhenFinished = removeJobWhenFinished;
	}
	
	private int status;
	/**
		Get the status of the job. 1 = waiting, 2 = running, 3 = completed ok, 4 = error
		@hibernate.property column="`status`" type="int" not-null="true"
	*/
	public int getStatus()
	{
		return status;
	}
	public void setStatus(int status)
	{
		this.status = status;
	}

	private boolean dryRun;
	/**
		If the job should be/was executed as a dry-run job.
		@hibernate.property column="`dry_run`" type="boolean" not-null="true"
	*/
	public boolean isDryRun()
	{
		return dryRun;
	}
	public void setDryRun(boolean dryRun)
	{
		this.dryRun = dryRun;
	}
	
	/**
		The maximum allowed length of the status message.
	*/
	public static final int MAX_STATUS_MESSAGE_LENGTH = 65535;
	private String statusMessage;
	/**
		Get a status message.
		@hibernate.property column="`status_message`" type="text" not-null="false"
	*/
	public String getStatusMessage()
	{
		return statusMessage;
	}
	public void setStatusMessage(String statusMessage)
	{
		this.statusMessage = statusMessage;
	}
	
	/**
		The maximum allowed length of the stack trace message.
	*/
	public static final int MAX_STACK_TRACE_LENGTH = 65535;
	private String stackTrace;
	/**
		Get the stack trace in case of an error
		@hibernate.property column="`stack_trace`" type="text" not-null="false"
	*/
	public String getStackTrace()
	{
		return stackTrace;
	}
	public void setStackTrace(String stackTrace)
	{
		this.stackTrace = stackTrace;
	}
	
	private int estimatedExecutionTime;
	/**
		Get the estimated execution time of the job. 
		0 = < 1 minute, 1 = 1 - 10 minutes, 2 =  < 1 hour, 3 = > 1 hour
		@hibernate.property column="`execution_time`" type="int" not-null="true"
	*/
	public int getEstimatedExecutionTime()
	{
		return estimatedExecutionTime;
	}
	public void setEstimatedExecutionTime(int estimatedExecutionTime)
	{
		this.estimatedExecutionTime = estimatedExecutionTime;
	}

	private int percentComplete;
	/**
		If the job is running, how many percent has been completed.
		@hibernate.property column="`percent_complete`" type="int" not-null="true"
	*/
	public int getPercentComplete()
	{
		return percentComplete;
	}
	public void setPercentComplete(int percentComplete)
	{
		this.percentComplete = percentComplete;
	}

	private int priority;
	/**
		The jobs priority. A lower value means a higher priority.
		@hibernate.property column="`priority`" type="int" not-null="true"
	*/
	public int getPriority()
	{
		return priority;
	}
	public void setPriority(int priority)
	{
		this.priority = priority;
	}

	private int activeProjectId;
	/**
		The ID of the project that should be made active when running this job.
		@hibernate.property column="`project_id`" type="int" not-null="true"
	*/
	public int getActiveProjectId()
	{
		return activeProjectId;
	}
	public void setActiveProjectId(int activeProjectId)
	{
		this.activeProjectId = activeProjectId;
	}
	
	private Date created;
	/**
		Get the date and time the job was created.
		@hibernate.property column="`created`" type="timestamp" not-null="true" update="false"
	*/
	public Date getCreated()
	{
		return created;
	}
	public void setCreated(Date created)
	{
		this.created = created;
	}

	private Date started;
	/**
		Get the date and time the job was started or null if the job hasn't been started.
		@hibernate.property column="`started`" type="timestamp" not-null="false"
	*/
	public Date getStarted()
	{
		return started;
	}
	public void setStarted(Date started)
	{
		this.started = started;
	}
	
	private Date ended;
	/**
		Get the date and time the job was ended or null if the job hasn't ended.
		@hibernate.property column="`ended`" type="timestamp" not-null="false"
	*/
	public Date getEnded()
	{
		return ended;
	}
	public void setEnded(Date ended)
	{
		this.ended = ended;
	}
	
	/**
		The maximum allowed length of the server name.
	*/
	public static final int MAX_SERVER_LENGTH = 255;
	private String server;
	/**
		The name of the server where the job is executing.
		@hibernate.property column="`server`" type="string" length="255" not-null="false"
	*/
	public String getServer()
	{
		return server;
	}
	public void setServer(String server)
	{
		this.server = server;
	}

	private Integer contextItem;
	/**
		The item type of the context that was active when configuring
		this job. null = no context was known
		@hibernate.property column="`context_item`" type="int"
		@since 2.8
	*/
	public Integer getContextItem()
	{
		return contextItem;
	}
	/**
		@since 2.8
	*/
	public void setContextItem(Integer contextItem)
	{
		this.contextItem = contextItem;
	}
	
	private Integer contextType;
	/**
		The context type that was active when configuring
		this job. 0 = single item, 1 = list
		@hibernate.property column="`context_type`" type="int"
		@since 2.8
	*/
	public Integer getContextType()
	{
		return contextType;
	}
	/**
		@since 2.8
	*/
	public void setContextType(Integer contextType)
	{
		this.contextType = contextType;
	}
	
	/**
		The maximum allowed length of the subcontext name.
		@since 2.8
	*/
	public static final int MAX_SUBCONTEXT_LENGTH = 255;
	private String subContext;
	/**
		The name of the subcontext that was active whjen configuring
		this job.
		@hibernate.property column="`subcontext`" type="string" length="255" not-null="false"
		@since 2.8
	*/
	public String getSubContext()
	{
		return subContext;
	}
	/**
		@since 2.8
	*/
	public void setSubContext(String subContext)
	{
		this.subContext = subContext;
	}
	
	/**
		The maximum allowed length of the signal transporter string.
	*/
	public static final int MAX_SIGNAL_TRANSPORTER_LENGTH = 655536;
	private String signalTransporter;
	/**
		Identifies the signal transporter to use. This is a string with 
		two parts separated by colon (:). The first part is the class name
		of the signal transporter class, the second part is the initialisation
		string for the transporter.
		@hibernate.property column="`signal_transporter`" type="text" not-null="false"
		@since 2.6
	*/
	public String getSignalTransporter()
	{
		return signalTransporter;
	}
	public void setSignalTransporter(String signalTransporter)
	{
		this.signalTransporter = signalTransporter;
	}
	
	private Integer jobAgentId;
	/**
		The ID of the job agent this job should run/is running/was running on.
		We store this as an ID value only and not a reference to {@link JobAgentData}
		since it would then be impossible to delete a job agent without also deleting
		all jobs that has been running on it.
		@since 2.8
		@hibernate.property column="`jobagent_id`" type="int" not-null="false"
	*/
	public Integer getJobAgentId()
	{
		return jobAgentId;
	}
	public void setJobAgentId(Integer jobAgentId)
	{
		this.jobAgentId = jobAgentId;
	}
	
	private Map<String, ParameterValueData<?>> parameters;
	/**
		The parameters for this job.
		@hibernate.map table="`JobParameters`" lazy="true" cascade="all"
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-index column="`name`" type="string" length="255"
		@hibernate.collection-many-to-many column="`value_id`" class="net.sf.basedb.core.data.ParameterValueData"
	*/
	public Map<String, ParameterValueData<?>> getParameters()
	{
		if (parameters == null) parameters = new HashMap<String, ParameterValueData<?>>();
		return parameters;
	}
	void setParameters(Map<String, ParameterValueData<?>> parameters)
	{
		this.parameters = parameters;
	}

	private Set<MessageData> messages;
	/**
		This is the inverse end.
		@see MessageData#getJob()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`job_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.MessageData"
	*/
	Set<MessageData> getMessages()
	{
		return messages;
	}
	void setMessages(Set<MessageData> messages)
	{
		this.messages = messages;
	}


}
