/*
	$Id: DataFileHandler.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.filehandler;

import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreEnabled;

/**
	This interface is the super-interface for all interfaces that
	are work on files of a specific {@link DataFileType}. Implementors
	of this interface should extend the {@link AbstractDataFileHandler}
	class since we may add more methods in the future. New methods will
	get default implementations in the <code>AbstractDataFileHandler</code>
	class.

	@author Nicklas
	@version 2.5
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface DataFileHandler
{

	/**
		Sets the item that we are currently working with.
		@param item The item
	*/
	public void setItem(FileStoreEnabled item);
	
	/**
		Set the file that is part of the file set with a {@link DataFileType}
		that is using this file handler. If the file set contains more 
		than one file with the same data handler this method is called
		one time for each member.
		@param member The member of the file set
	*/
	public void setFile(FileSetMember member);

}
