/*
	$Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.plugin;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.ProgressReporter;

import java.io.InputStream;

/**
	This interface should be implemented by an import
	plugin that supports auto-detection of the file
	format.
	<p>
	The plugin will be loaded and initialised normally, but
	instead of the {@link #run(Request, Response, ProgressReporter)} method 
	the methods in this interface are used. There are some additional
	requirements for an auto-detecting plugin:
	<ul>
	<li>All data must be in a single file
	<li>No additional runtime parameters are supported
	</ul>
	<p>
	Note that it is only if the user chooses to use the auto-detect
	function in the (web) client application that the methods in this
	interface are used. If the user manually selects to use a specific
	plugin the normal {@link #run(Request, Response, ProgressReporter)} 
	method is used. In other words, the plugin must support both methods of importing.

	@base.modified $Date$
	@author Enell
	@version 2.0
*/
public interface AutoDetectingImporter 
	extends Plugin
{
	/**
		Check if the given <code>InputStream</code> can be imported by this
		plugin.
		
		@param in The input stream to check
		@return TRUE if the stream can be imported, FALSE otherwise
		@throws BaseException If something goes wrong
	*/
	public boolean isImportable(InputStream in)
		throws BaseException;

	/**
		Import the data from the given <code>InputStream</code>.
	
		@param in The input stream to read from
	 	@param progress Progress reporter used by the caller to keep track of the progress. 
	 		Null is allowed 
		@throws BaseException If something goes wrong
	*/
	public void doImport(InputStream in, ProgressReporter progress)
		throws BaseException;
}
