/**
	$Id: AutoJoiner.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

/**
	An <code>AutoJoiner</code> is an object that automatically joins
	other tables in query depending on other query elements. For example,
	if we use the {@link Dynamic#select(net.sf.basedb.core.ExtraValue, String)} to select
	an extra value, the {@link net.sf.basedb.core.DynamicSpotQueryJoiners}
	makes sure that the table containing the extra values is joined to the
	query by calling the 
	{@link net.sf.basedb.core.DynamicSpotQuery#joinExtraValue(net.sf.basedb.core.ExtraValue, JoinType)}
	method.
	<p>
	How to programmatically join the correct table differs from query type to
	query type and from query element to query element. Each combination of
	query and query element usually needs it's own implementation of this interface.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public interface AutoJoiner<Q extends Query, E extends QueryElement>
{

	/**
		The type of the query the implementation works for. This can be an actual subclass
		or a superclass.
		@return A class object representing the query type
	*/
	public Class<Q> getQueryClass();

	/**
		The type of the query element the implementation works for. This can be an actual 
		subclass or a superclass.
		@return A class object representing the query element type
	*/
	public Class<E> getElementClass();
	
	/**
		Automatically join the tables that is is needed by the query element unless
		they are already joined.
		
		@param query The query 
		@param element The query element
	*/
	public void joinIfNeeded(Q query, E element);

}
