/*
	$Id: CountExpression.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;

/**
	Count the number of values: COUNT(e)

	@author Nicklas
	@version 2.0
	@see Aggregations#count(Expression, boolean)
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class CountExpression
	implements Expression 
{
	private final Expression e;
	private final boolean distinct;

	CountExpression(Expression e, boolean distinct)
	{
		this.e = e;
		this.distinct = distinct;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		return "COUNT(" + (distinct ? "DISTINCT " : "") + (e == null ? "*" : e.toQl(query, dc)) + ")";
	}
	/**
		@return The expression to count
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return e == null ? null : Collections.singleton(e);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return "COUNT(" + (distinct ? "DISTINCT " : "") + (e == null ? "*" : e ) + ")";
	}
	// -------------------------------------------
}

