/*
	$Id: SelectedExpression.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.dbengine.DbEngine;

/**
	An expression representing an already selected expression.
	If the select object has an alias, the alias is used otherwise the
	expression is regenerated.

	@author Nicklas
	@version 2.0
	@see Expressions#selected(Select)
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class SelectedExpression
	implements Expression
{

	private final Select select;

	SelectedExpression(Select select)
	{
		this.select = select;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		QuerySection section = query.getQuerySection();
		DbEngine engine = HibernateUtil.getDbEngine();
		String alias = select.getAlias();
		return alias != null && section.supportsColumnAlias(engine) ? 
				alias : select.getExpression().toQl(query, dc);
	}
	/**
		@return The selected expression
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(select);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		String alias = select.getAlias();
		return (alias != null ? alias : select.getExpression().toString());
	}
	// -------------------------------------------
}
