/*
	$Id: SqrtExpression.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Johan Enell, Nicklas Nordborg
	Copyright (C) 2007 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

package net.sf.basedb.core.query;

import java.util.Collection;
import java.util.Collections;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HibernateUtil;

/**
	 @author enell
	 @version 2.0
	 @base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/

public class SqrtExpression
	implements Expression
{
	private final Expression e1;

	SqrtExpression(Expression e1)
	{
		assert e1 != null : "e1 == null";
		this.e1 = e1;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	public String toQl(Query query, DbControl dc) 
		throws BaseException
	{
		String argument = e1.toQl(query, dc);
		String sqrt = "SQRT("+argument+")";
		if (HibernateUtil.getDbEngine().checkForInvalidNumberOperation())
		{
			sqrt = "(CASE WHEN " + argument + "<0 THEN null ELSE " + sqrt + " END)";
		}
		return sqrt;
	}
	/**
		@return The expression to take the square root of
		@since 2.0.1
	*/
	public Collection<? extends QueryElement> getChildren()
	{
		return Collections.singleton(e1);
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return "sqrt(" + e1 + ")";
	}
	// -------------------------------------------

}
