/**
	$Id: WhenStatement.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.core.query;

import net.sf.basedb.core.InvalidUseOfNullException;

/**
 	Create a WHEN statement to be used in a {@link CaseExpression}.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see Expressions#caseWhen(Expression, WhenStatement[])
*/
public class WhenStatement
{

	private final Restriction condition;
	private final Expression value;
	

	/**
		Create a new <code>WhenStatement</code>

		@param condition The condition that is controlling if this
			value is returned or not, null is not allowed
		@param value The value to return if the condition is TRUE, 
			null is not allowed
	*/
	public WhenStatement(Restriction condition, Expression value)
	{
		if (condition == null) throw new InvalidUseOfNullException("condition");
		if (value == null) throw new InvalidUseOfNullException("value");
		this.condition = condition;
		this.value = value;
	}
	
	/**
		Get the condition of this statement.
		@return A <code>Restriction</code> object
	*/
	public Restriction getCondition()
	{
		return condition;
	}
	
	/**
		Get the value returned by this statement
		@return An <code>Expression</code> object
	*/
	public Expression getValue()
	{
		return value;
	}
	
	
	
}
