/**
	$Id: EventType.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.extensions.events;

import net.sf.basedb.util.extensions.Extension;
import net.sf.basedb.util.extensions.ExtensionPoint;
import net.sf.basedb.util.extensions.Registry;

/**
	Represents different types of event. Some event types has been defined
	as static members of this class. External applications can create
	their own event types and send to registered extensions by
	calling {@link Registry#handleEvent(EventType, ExtensionPoint, Extension)}.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class EventType
{

	/**
		This event is sent after an extension or extension point has been
		registered with a registry.
	*/
	public static final EventType AFTER_REGISTRATION = new EventType("After registration");
	
	/**
		This event is sent during an update of an existing extension point or
		extension. The arguments to the {@link EventHandler#handleEvent(EventType, ExtensionPoint, Extension)}
		method is the currently registered extension point and extension. This
		event will be followed by {@link #AFTER_UPDATE}.
	*/
	public static final EventType BEFORE_UPDATE = new EventType("Before update");
	
	/**
		This event is sent during an update of an existing extension point or
		extension. The arguments to the {@link EventHandler#handleEvent(EventType, ExtensionPoint, Extension)}
		method is the new extension point and extension. This event is preceeded by
		{@link #BEFORE_UPDATE}.
	*/
	public static final EventType AFTER_UPDATE = new EventType("After update");
	
	/**
		This event is sent before an extension point or extension is
		unregistered.
	*/
	public static final EventType BEFORE_UNREGISTRATION = new EventType("Before unregistration");
	
	private final String name;
	
	/**
		Create a new event type.
		@param name A display name to use in the {@link #toString()} method
	*/
	public EventType(String name)
	{
		this.name = name;
	}
	
	@Override
	public String toString()
	{
		return name;
	}
	
}
