/**
	$Id: BooleanFormatter.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.formatter;

import net.sf.basedb.util.Values;

/**
	Formats a boolean value. It is possible to select between yes/no, true/false, and 1/0
	mappings.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class BooleanFormatter
	implements Formatter<Boolean>
{

	private final Type type;

	/**
		Create a new boolean formatter.
		@param type The type of output
	*/
	public BooleanFormatter(Type type)
	{
		this.type = type;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	public String format(Boolean value)
	{
		return value == null ? "" : type.getString(value);
	}
	public Boolean parseString(String value)
	{
		return Values.getBoolean(value);
	}
	// -------------------------------------------
	
	/**
		Enum for holding the various types of boolean output.
	*/
	public enum Type
	{
		/**
			Display yes/no.
		*/
		YES_NO("yes", "no"),

		/**
			Display true/false.
		*/
		TRUE_FALSE("true", "false"),

		/**
			Display 1/0.
		*/
		ONE_ZERO("1", "0");
		
		private final String trueValue;
		private final String falseValue;
		
		Type(String trueValue, String falseValue)
		{
			this.trueValue = trueValue;
			this.falseValue = falseValue;
		}
		
		public String getString(boolean value)
		{
			return value ? trueValue : falseValue;
		}
	}
}
