/**
	$Id: DateFormatter.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.formatter;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import net.sf.basedb.core.InvalidDataException;


/**
	Format a date for output in a client application. This implementation
	uses the {@link SimpleDateFormat} standard formatting routines.
	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see java.text.SimpleDateFormat
*/
public class DateFormatter
	implements Formatter<Date>
{

	private DateFormat dateFormat;
	private String format;
	
	/**
		Create a new date formatter.
		@param dateFormat The date format to use when formatting
	*/
	public DateFormatter(DateFormat dateFormat)
	{
		this.dateFormat = dateFormat;
		this.format = null;
	}
	
	/**
		Create a new date formatter.
		@param format The date format to use when formatting
		@see SimpleDateFormat
	*/
	public DateFormatter(String format)
	{
		this.dateFormat = new SimpleDateFormat(format);
		this.dateFormat.setLenient(true);
		this.format = format;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	public String format(Date value)
	{
		return value == null ? "" : dateFormat.format(value);
	}
	public Date parseString(String value)
	{
		if (value == null) return null;
		try
		{
			return dateFormat.parse(value);
		}
		catch (ParseException ex)
		{
			try
			{
				return new Date(new Long(value));
			}
			catch (Exception ex2)
			{
				throw new InvalidDataException(value + " is not a valid date for format: " + 
					format, ex);
			}
		}
	}
	// -------------------------------------------

	/**
		Get access to the underlying {@link DateFormat} object
		used for formatting and parsing dates.
		@since 2.8
	*/
	public DateFormat getDateFormat()
	{
		return dateFormat;
	}
	
}
