/**
	$Id: LineNoFunction.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.parser;

import java.util.Stack;

import org.nfunk.jep.ParseException;

import net.sf.basedb.util.jep.JepFunction;

/**
	Adds a <code>lineNo()</code> function to Jep. The lineNo() function is used with the 
	{@link FlatFileParser} to get the current line number. Use the 
	{@link FlatFileParser#getMapper(String)} with an argument starting with an equal
	sign to create a mapper that uses Jep.
	<pre class="code">
FlatFileParser ffp = ...
Mapper lineNoMapper = ffp.getMapper("=lineNo()");
while (ffp.hasMoreData())
{
	Data data = ffp.nextData();
	int lineNumber = lineNoMapper.getInt(data);
}
</pre>

	@author nicklas
	@version 2.4
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class LineNoFunction
	implements JepFunction
{
	
	private FlatFileParser.Data data;
	
	public LineNoFunction()
	{}
	
	/*
		From the JepFunction interface
		-------------------------------------------
	*/
	/**
		@return The string "lineNo"
	*/
	public String getFunctionName()
	{
		return "lineNo";
	}
	// -------------------------------------------
	/*
		From the PostfixMathCommandI interface
		-------------------------------------------
	*/
	/**
		@return Always 0
	*/
	public int getNumberOfParameters()
	{
		return 0;
	}
	public void setCurNumberOfParameters(int n)
	{}
	public boolean checkNumberOfParameters(int n)
	{
		return n == 0;
	}
	@SuppressWarnings({"unchecked"})
	public void run(Stack stack)
		throws ParseException
	{
		if (stack == null) 
		{
			throw new ParseException("Stack is null");
		}
		stack.push(data.lineNo());
	}
	// -------------------------------------------

	public void setData(FlatFileParser.Data data)
	{
		this.data = data;
	}

}
