/**
	$Id: NullIfExceptionMapper.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.parser;

import net.sf.basedb.util.parser.FlatFileParser.Data;

/**
 	A mapper that returns null if there is an exception when parsing the
 	input data. The actual mapping is delegated to a parent mapper.

	@author nicklas
	@version 2.4
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
*/
public class NullIfExceptionMapper
	implements Mapper
{

	private final Mapper parent;
	
	/**
		Create a new CropStringMapper.
		@param parent The parent mapper that does the actual mapping
	*/
	public NullIfExceptionMapper(Mapper parent)
	{
		this.parent = parent;
	}

	/*
		From the Mapper interface
		-------------------------------------------
	*/
	public String getValue(Data data)
	{
		try
		{
			return parent.getValue(data);
		}
		catch (Throwable t)
		{}
		return null;
	}
	public Integer getInt(Data data)
	{
		try
		{
			return parent.getInt(data);
		}
		catch (Throwable t)
		{}
		return null;
	}
	public Float getFloat(Data data)
	{
		try
		{
			return parent.getFloat(data);
		}
		catch (Throwable t)
		{}
		return null;
	}
	// -------------------------------------------

}
