/*
	$Id: PlotAnnotation.java 4479 2008-09-05 15:16:27Z jari $

	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.util.plot;

import java.util.Arrays;
import java.util.Collection;

import net.sf.basedb.core.query.SqlResultIterator;

/**
	Used to map spot data into different annotations. The mapping is done by checking
	which {@link #getValues()} collection a value falls in.

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:27 +0200 (Fri, 05 Sep 2008) $
	@see ScatterPlot#addData(SqlResultIterator, Collection)
*/
public class PlotAnnotation
	implements Comparable
{
	
	private final String name;
	private final Collection<Integer> values;
	
	/**
		Create a new PlotAnnotation.
		@param name The name of the annotation, which will be used as a label in the image
		@param value The value that belongs to this annotation
		@since 2.2
	*/
	public PlotAnnotation(String name, int value)
	{
		this(name, Arrays.asList(value));
	}
	
	/**
		Create a new PlotAnnotation.
		@param name The name of the annotation, which will be used as a label in the image
		@param values The values that belongs to this annotation
	*/
	public PlotAnnotation(String name, Collection<Integer> values)
	{
		this.name = name;
		this.values = values;
	}
	
	/**
		The name of the annotation, which will be used as a label in the image.
	*/
	public String getName()
	{
		return name;
	}
	
	/**
		The values that falls into this annotation. This collection shouldn't contain
		a value that is also found in another <code>PlotAnnotation</code>.
	*/
	public Collection<Integer> getValues()
	{
		return values;
	}

	public int compareTo(Object o)
	{
		return this.name.compareTo(((PlotAnnotation)o).name);
	}
	
}
