/**
$Id: FilePacker.java 4483 2008-09-07 21:59:52Z jari $

Copyright (C) 2007 Nicklas Nordborg, Martin Svensson
Copyright (C) 2008 Jari Hakkinen

This file is part of BASE - BioArray Software Environment.
Available at http://base.thep.lu.se/

BASE is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

BASE is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

package net.sf.basedb.util.zip;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 	This interface should be implemented by classes that can pack files and directories
 	from BASE file system. The implementing class must also have a public no-argument
 	constructor. The implementing classes are used by the  
 	{@link net.sf.basedb.plugins.PackedFileExporter} plug-in. Each class should
 	be registered in a {@link net.sf.basedb.core.PluginConfiguration} for the
 	<code>PackedFileExporter</code>.
 	
  	@author Martin, Nicklas
  	@version 2.4  	
  	@base.modified $Date: 2006-11-30 10:50:05 +0100 (Thu, 30 Nov 2006) $
*/
public interface FilePacker
{
	
	/**
		Get a short description of this file format that is suitable for display in
		selection list boxes.
	*/
	public String getDescription();
	
	/**
	 	Gets the extension that files, packed with this tool, should have,
	 	for example, <code>zip</code> or <code>tar.gz</code> Don't include the first 
	 	dot!
	  	@return A String with the file extension excluding the dot
	*/
	public String getFileExtension();
	
	/**
		Gets the MIME type to give compressed files from this packer
	  	@return The MIME type of the compressed file format
	*/
	public String getMimeType();
	
	/**
		The output stream that the compressed files should be written to.
		@param out The output stream to write to
		@throws IOException If there is an error setting the output stream
	*/
	public void setOutputStream(OutputStream out)
		throws IOException;
	
	/**
	 	Compress the uncompressed input stream to an entry with the given
	 	name into the outputstream.
	 	@param entryName The name of the packed resource, including path 
	 		information
	  	@param in The input stream to read uncompressed data from, or null
	  		if the entry represents a directory
	  	@param size The number of bytes of uncompressed data
	  	@param lastModified The time the contents was last modified, or 0
	  		if not known
	  	@throws IOException If there are any errors when reading or writing
	*/
	public void pack(String entryName, InputStream in, long size, long lastModified)
		throws IOException;
	
	/**
		End the packing and close. The plug-in should close any open resources, including
		the output stream set by the {@link #setOutputStream(OutputStream)} method, and
		cleanup temporary objects.
		
	  	@throws IOException If there are any errors
	*/
	public void close()
		throws IOException;
}
