/**
	$Id: ExperimentInfo.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

package net.sf.basedb.info;


import java.io.Serializable;
import java.util.Date;

/**
 	Holds information about an Experiment item from BASE. 
 	This object can be used to transfer information about 
 	experiment through BASE web services. Only simple properties
 	like id, name, texts etc. can be put in this info-object.
 	 
  	@author Martin
  	@version 2.5
 */
public class ExperimentInfo 
	extends CommonItemInfo
		implements Serializable
{
	private static final long serialVersionUID = 7973320584668351841L;
	private String abstractText = null;	
	private String affiliations = null;
	private String authors = null;
	private String experimentDesign = null;
	private String experimentType = null;
	private String publication = null;
	private String title = null;
	private Date publicationDate = null;
	private String pubMedId = null;
		
	/**
	 	Creates an info-object with no properties set.
	 */
	public ExperimentInfo()
	{
		super();
	}

	/**	
	  	@param abstractText The abstractText to set.
	*/
	public void setAbstractText(String abstractText)
	{
		this.abstractText = abstractText;
	}

	/**	
	 	@return Returns the abstractText.
	*/
	public String getAbstractText()
	{
		return abstractText;
	}

	/**	
	  	@param affiliations The affiliations to set.
	*/
	public void setAffiliations(String affiliations)
	{
		this.affiliations = affiliations;
	}

	/**	
	 	@return Returns the affiliations.
	*/
	public String getAffiliations()
	{
		return affiliations;
	}

	/**	
	  	@param authors The authors to set.
	*/
	public void setAuthors(String authors)
	{
		this.authors = authors;
	}

	/**	
	 	@return Returns the authors.
	*/
	public String getAuthors()
	{
		return authors;
	}

	/**	
	  	@param experimentDesign The experimentDesign to set.
	*/
	public void setExperimentDesign(String experimentDesign)
	{
		this.experimentDesign = experimentDesign;
	}

	/**	
	 	@return Returns the experimentDesign.
	*/
	public String getExperimentDesign()
	{
		return experimentDesign;
	}

	/**	
	  	@param experimentType The experimentType to set.
	*/
	public void setExperimentType(String experimentType)
	{
		this.experimentType = experimentType;
	}

	/**	
	 	@return Returns the experimentType.
	*/
	public String getExperimentType()
	{
		return experimentType;
	}

	/**	
	  	@param publication The publication to set.
	*/
	public void setPublication(String publication)
	{
		this.publication = publication;
	}

	/**	
	 	@return Returns the publication.
	*/
	public String getPublication()
	{
		return publication;
	}

	/**	
	  	@param title The title to set.
	*/
	public void setTitle(String title)
	{
		this.title = title;
	}

	/**	
	 	@return Returns the title.
	*/
	public String getTitle()
	{
		return title;
	}

	/**	
	  	@param publicationDate The publicationDate to set.
	*/
	public void setPublicationDate(Date publicationDate)
	{
		this.publicationDate = publicationDate;
	}

	/**	
	 	@return Returns the publicationDate.
	*/
	public Date getPublicationDate()
	{
		return publicationDate;
	}

	/**	
	  	@param pubMedId The pubMedId to set.
	*/
	public void setPubMedId(String pubMedId)
	{
		this.pubMedId = pubMedId;
	}

	/**	
	 	@return Returns the pubMedId.
	*/
	public String getPubMedId()
	{
		return pubMedId;
	}
}
