/*
 $Id: Base1PluginExecuter.java 4531 2008-09-17 12:27:26Z nicklas $

 Copyright (C) 2006 Johan Enell, Jari Hakkinen, Nicklas Nordborg
 Copyright (C) 2007 Johan Enell, Nicklas Nordborg
 
 This file is part of BASE - BioArray Software Environment.
 Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
*/

package net.sf.basedb.plugins;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.BooleanParameterType;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.DynamicQuery;
import net.sf.basedb.core.DynamicResultIterator;
import net.sf.basedb.core.DynamicSpotQuery;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.ExtraValue;
import net.sf.basedb.core.ExtraValueType;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.FloatParameterType;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.Include;
import net.sf.basedb.core.IntegerParameterType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.MappingBatcher;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.PathParameterType;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.PositionBatcher;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.ReporterBatcher;
import net.sf.basedb.core.ReporterList;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.SpotBatcher;
import net.sf.basedb.core.SpotExtraValueBatcher;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.Transformation;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.User;
import net.sf.basedb.core.Version;
import net.sf.basedb.core.VirtualColumn;
import net.sf.basedb.core.VirtualTable;
import net.sf.basedb.core.data.RawData;
import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.AbstractAnalysisPlugin;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.InteractivePlugin;
import net.sf.basedb.core.plugin.Permissions;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.core.query.Dynamic;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.JoinType;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.SqlResult;
import net.sf.basedb.core.signal.SignalException;
import net.sf.basedb.core.signal.SignalHandler;
import net.sf.basedb.core.signal.SignalTarget;
import net.sf.basedb.core.signal.ThreadSignalHandler;
import net.sf.basedb.util.Diff3;
import net.sf.basedb.util.Enumeration;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.XMLUtil;
import net.sf.basedb.util.parser.FlatFileParser;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.filter.Filter;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.sql.SQLException;
import java.util.AbstractSet;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
	This plug-in executes a plug-in from the Base1 system. It takes a base1 configuration
	file to create its configuration.
	
	@author Johan Enell
	@version 2.2
	@base.modified $Date: 2008-09-17 14:27:26 +0200 (Wed, 17 Sep 2008) $
*/
public class Base1PluginExecuter 
	extends AbstractAnalysisPlugin 
	implements InteractivePlugin, SignalTarget
{

	/**
		Plugin description
	*/
	private static final About about = new AboutImpl(
		"Base1PluginExecuter",
		"This plug-in simulates the plug-in runner from Base 1.2 to enable old plug-ins to be executed.",
		Version.getMajor() + "." + Version.getMinor() + "." + Version.getMaintenance(),
		"2006, Base 2 development team",
		null,
		null,
		"http://base.thep.lu.se");
 
	/**
		The plug-in permissions.
	*/
	private static final Set<Permissions> permissions = new HashSet<Permissions>();
	
	private static final URL dtdFile = Base1PluginExecuter.class.getResource("/net/sf/basedb/core/dtd/base1-plugin-configuration-file.dtd");
	
	/* Configuration parameters*/

	/**
		Parameter definition that asks for the configuration file that should be imported.
	*/
	private static final PluginParameter<File> fileParameter = new PluginParameter<File>(
		"file",
		"File",
		"The file to import the data from. If you leave this parameter blank you can manually configure the plug-in.",
		new FileParameterType(null, false, 1));

	/**
		The path to where the plug-in executer can find the executables. In Base1 this was 
		default to /usr/local/base/plugins.
	*/
	private static PluginParameter<String> execPathParameter = new PluginParameter<String>(
		"execPath",
		"Plugin executables path",
		"The path to the executables",
		new StringParameterType(255, null, true));

	/**
		This parameter describes if the data should be exported in serial format or
		in matrix.
	*/
	private PluginParameter<Boolean> serialFormatParameter;
	
	/**
		The version number of the plug-in.
	*/
	private PluginParameter<String> versionNumberParameter;
	
	/**
		The name of the executable.
	*/
	private PluginParameter<String> execNameParameter;
	
	/**
		If the data should be exported averaged per gene.
	*/
	private PluginParameter<Boolean> geneAveragesParameter;
	
	/**
		The minimum number of channels. 
	*/
	private PluginParameter<Integer> minChannelsParameter;
	
	/**
		The maximum number of channels, 0 means unlimited.
	*/
	private PluginParameter<Integer> maxChannelsParameter;
	
	/**
		If the plug-in should leave the stdin.txt after execution.
	*/
	private PluginParameter<Boolean> leaveStdinParameter;
	
	/**
		If the plug-in should leave the stdout.txt after execution.
	*/
	private PluginParameter<Boolean> leaveStdoutParameter;
	
	/**
		The reporter fields that should be exported as a tab separated string
		of column headers.
	*/
	private PluginParameter<String> usedColumnsParameter;
	
	/**
		The spot fields that should be exported as a tab separated string of 
		column headers.
	*/
	private PluginParameter<String> usedFieldsParameter;
	
	/**
		This parameter describes the job parameters that the base1 plugin needs. 
	*/
	private PluginParameter<String> jobParametersParameter = new PluginParameter<String>(
		"jobParameters",
		"Job parameters",
		"An XML text that describes the job parameters.",
		new StringParameterType(null, null, true));
	
	/* Job parameters*/
	/**
		The directory where the plug-in will store it's output.
	*/
	private PluginParameter<String> pluginDirectoryParameter;

	/**
		This command is used if the user decides to configure the plugin by hand.
	*/
	private static final String COMMAND_MANUAL_CONFIG = "COMMAND_MANUAL_CONFIG";

	/**
		Holds info about a job configuration request.
	*/
	private RequestInformation configureJob;
	
	/**
		Holds information about a plugin configuration request.
	*/
	private RequestInformation configurePlugin;
	
	/**
		Holds information about a manual plugin configuration request.
	*/
	private RequestInformation configureManual;

	/**
		This list holds the job parameters derived from the configuration file.
	*/
	private List<PluginParameter<?>> jobParameters;

	/**
		This is the directory where the plugin will generate its output.
	*/
	private java.io.File execDirectory;

	private int pluginDirectoryId = -1;
	
	private ThreadSignalHandler signalHandler;

	/* From InteractivePlugin interface*/

	/**
		Checks if the BioAssaySet has the number of channels specified by the 
		configuration and that it has the extra values.
		
		@param context The context from where the plugin is invoked.
		@param item The item that should be checked if it is in context.
		@return null if the the item is in context, otherwise an error message.
		@see net.sf.basedb.core.plugin.AbstractAnalysisPlugin#isInContext(net.sf.basedb.core.plugin.GuiContext, java.lang.Object)
	*/
	@Override
	public String isInContext(GuiContext context, Object item)
	{
		String message = super.isInContext(context, item);
		if (message == null)
		{
			BioAssaySet bas = (BioAssaySet)item;
			if (bas.getNumSpots() <= 0)
			{
				message = "This plug-in requires spot data in the database";
			}
			else
			{
				DbControl dc = null;
				try
				{
					dc = sc.newDbControl();
					getManualConfigureParameters();
	
					int channels = bas.getRawDataType().getChannels();
					int maxChannels = Values.getInt(String.valueOf(configuration.getValue(maxChannelsParameter.getName())), 0);
					int minChannels = Values.getInt(String.valueOf(configuration.getValue(minChannelsParameter.getName())), 0);
					if (channels < minChannels || ( maxChannels != 0 && channels > maxChannels))
					{
						return configuration.getPluginConfiguration().getName() + " can't work on " + channels + "-channel data; min=" + minChannels + "; max=" + maxChannels;
					}
					List<String> extraValues = new ArrayList<String>();
					for (ExtraValue ev : bas.getExtraValues().list(dc))
					{
						extraValues.add(ev.getExtraValueType().getExternalId());
					}
					String[] usedFields = String.valueOf(configuration.getValue(usedFieldsParameter.getName())).split("\\\\t");
					for (String field : usedFields)
					{
						if (field.startsWith("_xc_") && !extraValues.contains(field.substring(4)))
						{
							return "Requires extra value '"+ field.substring(4) +"' attached to BioAssaySet";
						}
					}
				}
				finally
				{
					if (dc != null) dc.close();
				}
			}
		}
		return message;
	}

	public RequestInformation getRequestInformation(GuiContext context, String command)
		throws BaseException
	{
		RequestInformation requestInformation = null;

		if (command.equals(Request.COMMAND_CONFIGURE_PLUGIN))
		{
			requestInformation = getConfigurePluginParameters();
		}
		else if (command.equals(Base1PluginExecuter.COMMAND_MANUAL_CONFIG))
		{
			requestInformation = getManualConfigureParameters();
		}
		else if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{
			requestInformation = getConfigureJobParameters();
		}
		else
		{
			throw new BaseException("Unknown command: "+command);
		}
		return requestInformation;
	}
	
	
	public void configure(GuiContext context, Request request, Response response)
	{
		String command = request.getCommand();
		DbControl dc = null;
		try
		{
			if (command.equals(Request.COMMAND_CONFIGURE_PLUGIN))
			{
				storeValue(configuration, request, execPathParameter);
				
				dc = sc.newDbControl();
				File f = (File) request.getParameterValue(fileParameter.getName());
				if (f != null)
				{
					if (f.isInDatabase()) f = File.getById(dc, f.getId());
					
					getManualConfigureParameters();
					Map<String, String> headers = getParametersFromBaseFile(f);
					configuration.setValue(serialFormatParameter.getName(), serialFormatParameter.getParameterType(), getBoolean(headers.get(serialFormatParameter.getName())));
					configuration.setValue(versionNumberParameter.getName(), versionNumberParameter.getParameterType(), headers.get(versionNumberParameter.getName()));
					configuration.setValue(execNameParameter.getName(), execNameParameter.getParameterType(), headers.get(execNameParameter.getName()));
					configuration.setValue(geneAveragesParameter.getName(), geneAveragesParameter.getParameterType(), getBoolean(headers.get(geneAveragesParameter.getName())));
					configuration.setValue(minChannelsParameter.getName(), minChannelsParameter.getParameterType(), new Integer(headers.get(minChannelsParameter.getName())));
					configuration.setValue(maxChannelsParameter.getName(), maxChannelsParameter.getParameterType(), new Integer(headers.get(maxChannelsParameter.getName())));
					configuration.setValue(leaveStdinParameter.getName(), leaveStdinParameter.getParameterType(), getBoolean(headers.get(leaveStdinParameter.getName())));
					configuration.setValue(leaveStdoutParameter.getName(), leaveStdoutParameter.getParameterType(), getBoolean(headers.get(leaveStdoutParameter.getName())));
					configuration.setValue(usedColumnsParameter.getName(), usedColumnsParameter.getParameterType(), headers.get(usedColumnsParameter.getName()));
					configuration.setValue(usedFieldsParameter.getName(), usedFieldsParameter.getParameterType(), headers.get(usedFieldsParameter.getName()));
					configuration.setValue(jobParametersParameter.getName(), jobParametersParameter.getParameterType(), headers.get(jobParametersParameter.getName()));
					PluginConfiguration config = configuration.getPluginConfiguration();
					if (config != null)
					{
						if (config.isInDatabase()) config = PluginConfiguration.getById(dc,config.getId());
						config.setDescription(headers.get("descr").replaceAll("\\\\r\\\\n", "\n"));
					}
					
					response.setDone("Configured plug-in with file.");
				}
				else
				{
					response.setContinue(Base1PluginExecuter.COMMAND_MANUAL_CONFIG);
				}
			}
			else if (command.equals(Base1PluginExecuter.COMMAND_MANUAL_CONFIG))
			{
				storeValue(configuration, request, serialFormatParameter);
				storeValue(configuration, request, versionNumberParameter);
				storeValue(configuration, request, execNameParameter);
				storeValue(configuration, request, geneAveragesParameter);
				storeValue(configuration, request, minChannelsParameter);
				storeValue(configuration, request, maxChannelsParameter);
				storeValue(configuration, request, leaveStdinParameter);
				storeValue(configuration, request, leaveStdoutParameter);
				storeValue(configuration, request, usedColumnsParameter);
				storeValue(configuration, request, usedFieldsParameter);
				storeValue(configuration, request, jobParametersParameter);
				// test the xml in the jobParameters parameters
				try
				{
					getJobParametersFromXML(String.valueOf(configuration.getValue(jobParametersParameter.getName())), false);
				}
				catch (BaseException e)
				{
					throw new BaseException("Error in the jobparameters xml. "+e.getMessage());
				}
				response.setDone("Plugin configuration done.");
			}
			else if (command.equals(Request.COMMAND_CONFIGURE_JOB))
			{
				storeValue(job, request, getSourceBioAssaySetParameter(null, null));
				for (PluginParameter<?> p : getJobParameters())
				{
					storeValue(job, request, p);
				}
				List<Throwable> errors = validateRequestParameters(getJobParameters(), request);
				if (errors == null)
					response.setDone("Job configuration complete", Job.ExecutionTime.SHORT);
				else
					response.setError(errors.size() + " invalid parameter(s) were found in the request", errors);
			}
			else
			{
				throw new BaseException("Command unknown: "+command);
			}
		}
		catch (Throwable ex)
		{
			response.setError(ex.getMessage(), Arrays.asList(ex));
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
	/**
		Copies the content from a File to a directory on the file system.
		
		@param f The file to be copied.
		@param d The directory where the file will be written.
		@throws IOException If there is any error with target.
	*/
	private void copy(File f, java.io.File d)
		throws IOException
	{
		if (f != null && d.isDirectory())
		{
			java.io.File target = new java.io.File(d, f.getName());
			InputStream in = f.getDownloadStream(0);
			FileOutputStream out = new FileOutputStream(target);
			
			StreamHandler sh = new StreamHandler(in, out);
			sh.run();
		}
	}
	
	
	private int createPluginDirectory()
	{
		DbControl dc = null;
		int id = -1;
		try
		{
			dc = sc.newDbControl();
			String pathName = (String) job.getValue(pluginDirectoryParameter.getName()) + "/" + job.getId();
			Path path = new Path(pathName, Path.Type.DIRECTORY);
			Directory jobDirectory = null;
			try
			{
				// Load existing...
				jobDirectory = Directory.getByPath(dc, path);
				// ...and remove files and sub directories in it
				emptyDir(dc, path);
			}
			catch(ItemNotFoundException e)
			{
				// Create a new directory
				jobDirectory = Directory.getNew(dc, path);
				dc.saveItem(jobDirectory);
			}
			dc.commit();
			id = jobDirectory.getId();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id; 
	}
	
	/**
		Deletes all files and sub directories under dir.
		
		@param dir
		@return true if all deletions were successful
	*/
	private boolean deleteDir(java.io.File dir)
	{
		if (dir.isDirectory())
		{
			java.io.File[] children = dir.listFiles();
			for (java.io.File child : children)
			{
				boolean success = deleteDir(child);
				if (!success)
				{
					return false;
				}
			}
		}
		return dir.delete();
	}
	
	
	/**
		Deletes all files and sub directories under dir. The directory given
		itself is not deleted.
		
		@param dc
		@param dir
		@return true if all deletions were successful
	*/
	private boolean emptyDir(DbControl dc, Path dir)
	{
		if (dir.getType() == Path.Type.DIRECTORY)
		{
			Directory d = Directory.getByPath(dc, dir);
			for (File f : d.getFiles().list(dc))
			{
				dc.deleteItem(f);
			}
			for (Directory subD : d.getSubDirectories().list(dc))
			{
				if (!emptyDir(dc, subD.getPath()))
				{
					return false;
				}
				dc.deleteItem(subD);
			}
			return true;
		}
		return false;
	}

	@Override
	public Collection<Permissions> getPermissions()
	{
		if (permissions.size() == 0)
		{
			permissions.add(new Permissions(Item.ANNOTATIONTYPE, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.DIRECTORY, null, EnumSet.of(Permission.CREATE, Permission.DELETE)));
			permissions.add(new Permissions(Item.EXPERIMENT, null, EnumSet.of(Permission.WRITE)));
			permissions.add(new Permissions(Item.FILE, null, EnumSet.of(Permission.CREATE, Permission.DELETE)));
			permissions.add(new Permissions(Item.PLUGINCONFIGURATION, EnumSet.of(Permission.WRITE), null));
			permissions.add(new Permissions(Item.RAWBIOASSAY, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.REPORTER, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.EXTRAVALUETYPE, null, EnumSet.of(Permission.CREATE, Permission.USE)));
			permissions.add(new Permissions(Item.USER, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.JOB, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.FORMULA, null, EnumSet.of(Permission.READ)));
			permissions.add(new Permissions(Item.REPORTERLIST, null, EnumSet.of(Permission.CREATE, Permission.USE)));
		}
		return permissions;
	}
	
	// ***

	// *** From Plugin interface

	public About getAbout()
	{
		return about;
	}
	

	@Override
	public boolean supportsConfigurations()
	{
		return true;
	}
	

	@Override
	public boolean requiresConfiguration()
	{
		return true;
	}
	

	public void run(Request request, Response response, ProgressReporter progress)
	{
		if (signalHandler != null) signalHandler.setWorkerThread(null);
		String command = request.getCommand();
		if (command.equals(Request.COMMAND_EXECUTE))
		{
			String msg = "";
			DbControl dc = null;
			File stdin = null;
			File stdout = null;
			Transformation trans = null;
			try
			{
				dc = sc.newDbControl();
				getManualConfigureParameters();
				getConfigureJobParameters();
	
				//Export
				try
				{
					BioAssaySet parentBas = getSourceBioAssaySet(dc);
					trans = parentBas.newTransformation(Job.getById(dc, job.getId()));
					trans.setName(PluginConfiguration.getById(dc, configuration.getId()).getName());
					dc.saveItem(trans);
					progress.display(0, "Exporting data to be used by plugin.");
					stdin = File.getById(dc, exportData());
					checkInterrupted();
					copy(stdin, getExecDirectory());
				}
				catch(Exception e)
				{
					response.setError("Error during export: " + e.getMessage(), Arrays.asList(e));
					return;
				}
				
				//Execution
				Process externalProcess = null;
				try
				{
					checkInterrupted();
					progress.display(10, "Running on remote computation server.");
					externalProcess = Runtime.getRuntime().exec(getExecLine(), null, getExecDirectory());
	
					ByteArrayOutputStream err = new ByteArrayOutputStream();
					StreamHandler errorStream = new StreamHandler(
							new BufferedInputStream(externalProcess.getErrorStream()), 
							new BufferedOutputStream(err));
	
					FileOutputStream out = 
						new FileOutputStream(new java.io.File(getExecDirectory(), "stdout.txt"));
					StreamHandler inputStream = new StreamHandler(
						new BufferedInputStream(externalProcess.getInputStream()), 
						new BufferedOutputStream(out));
					
					FileInputStream in = 
						new FileInputStream(new java.io.File(getExecDirectory(), "stdin.txt"));
					StreamHandler outputStream = new StreamHandler(
						new BufferedInputStream(in), 
						new BufferedOutputStream(externalProcess.getOutputStream()));
					
					inputStream.start();
					errorStream.start();
					outputStream.start();
					int exitValue = 0;
					try
					{
						exitValue = externalProcess.waitFor();
					}
					catch (InterruptedException ex)
					{
						// Aborted by user
						externalProcess.destroy();
						throw new SignalException("Aborted by user", ex);
					}

					progress.display(50, "Reading from error stream.");
					for(int i = 0; errorStream.isAlive(); ++i)
					{
						Thread.sleep(1000);
						if (i > 300) throw new BaseException("Waited 5 minutes for the errormessage. I give up.");
					}
										
					err.flush();
					msg = err.toString();
					out.flush();
					
					if (exitValue != 0)
					{
						response.setError(msg, null);
						return;
					}
				}
				catch (Exception e)
				{
					response.setError("Error during execution: " + e.getMessage(), Arrays.asList(e));
					return;
				}
				finally
				{
					//Copy files
					progress.display(60, "Copying files to server.");
					importFiles(dc, trans);
					if (!deleteDir(getExecDirectory()))
					{
						response.setError("Could not remove execution directory: "+getExecDirectory().getAbsolutePath(), null);
					}
					stdout = File.getFile(dc, getPluginDirectory(dc), "stdout.txt", true);
				}
				
				//Import data
				try
				{
					checkInterrupted();
					progress.display(70, "Importing data from plugin.");
					importData(dc, stdout, trans);
				}
				catch (Exception ex)
				{
					response.setError(ex.getMessage(), Arrays.asList(ex));
					return;
				}
				dc.commit();
			}
			finally
			{
				if (dc != null) dc.close();
			}
			
			//Clean up
			try
			{
				dc = sc.newDbControl();
				if (!Values.getBoolean(String.valueOf(configuration.getValue(leaveStdinParameter.getName()))))
				{
					if (stdin != null)
					{
						stdin = File.getById(dc, stdin.getId());
						if (trans != null)
						{
							trans = Transformation.getById(dc, trans.getId());
							AnyToAny ata = AnyToAny.getByName(dc, trans, stdin.getPath().toString());
							dc.deleteItem(ata);
						}
						dc.deleteItem(stdin);
					}
				}
				if (!Values.getBoolean(String.valueOf(configuration.getValue(leaveStdoutParameter.getName()))))
				{
					if (stdout != null)
					{
						stdout = File.getById(dc, stdout.getId());
						if (trans != null)
						{
							trans = Transformation.getById(dc, trans.getId());
							AnyToAny ata = AnyToAny.getByName(dc, trans, stdout.getPath().toString());
							dc.deleteItem(ata);
						}
						dc.deleteItem(stdout);
					}
				}
				dc.commit();
			}
			finally
			{
				if (dc != null) dc.close();
			}
			response.setDone(msg);
		}
		else
		{
			response.setError("Unknown command: " + command, null);
		}
	}
	/*
		From the SignalTarget interface
		-------------------------------------------
	*/
	public SignalHandler getSignalHandler()
	{
		signalHandler = new ThreadSignalHandler();
		return signalHandler;
	}
	// -------------------------------------------


	/**
		Export the bioassay set to a file called 'stdin.txt' in the plugin directory.
		
		@return The ID of the file where the data is exported to.
	*/
	private int exportData()
	{
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();
			Directory d = getPluginDirectory(dc);
			File file = File.getFile(dc, d, "stdin.txt", true);
			file.setMimeTypeAuto("text/plain", null);
			dc.saveItem(file);
			
			BioAssaySet bas = getSourceBioAssaySet(dc);
			
			Map<String, String> nameMap = loadNameMap(dc, bas);
			String usedColumns = (String) configuration.getValue(usedColumnsParameter.getName());
			List<String> reporterFields = usedColumns == null ?
				null : convertNames(nameMap, usedColumns.split("\\\\t"));
			
			String usedFields = (String) configuration.getValue(usedFieldsParameter.getName());
			List<String> spotFields = usedFields == null ?
				null : convertNames(nameMap, usedFields.split("\\\\t"));
			
			if (spotFields != null && spotFields.contains("_allIntensities"))
			{
				spotFields.remove("_allIntensities");
				int channels = bas.getRawDataType().getChannels();
				for (int i = 1; i <= channels; ++i)
				{
					spotFields.add("intensity"+i);
				}
			}
			
			Boolean mergeReporters = (Boolean) configuration.getValue(geneAveragesParameter.getName());
			
			TreeMap<String, JobParameter> base1parametertype = getJobParameterObjectsFromXML(String.valueOf(configuration.getValue(jobParametersParameter.getName())), true);
			ListMap<String, String> parameters = new ListMap<String, String>();
			for (PluginParameter<?> pp : getJobParametersFromXML(String.valueOf(configuration.getValue(jobParametersParameter.getName())), true))
			{
				String name = pp.getName();
				Object value = job.getValue(name);
				String valueString = (value == null) ? "" : value.toString();
				JobParameter jp=base1parametertype.get(name);
				// replacing line breaks for Base1 TEXT and STRING types
				if (jp.getType() == Base1JobParameterType.STRING ||
						jp.getType() == Base1JobParameterType.TEXT)
				{
					Pattern p=Pattern.compile("\r");
					Matcher m=p.matcher(valueString);
					valueString = m.replaceAll("\\\\r");
					m.usePattern(Pattern.compile("\n"));
					m.reset(valueString);
					valueString = m.replaceAll("\\\\n");
				}
				parameters.put(name, valueString);
			}
			
			checkInterrupted();
			BioAssaySetExporter exporter = new BioAssaySetExporter();
			if ((Boolean) configuration.getValue(serialFormatParameter.getName()))
			{
				exporter.exportBaseFileSerial(bas, file, parameters, reporterFields, spotFields, mergeReporters);
			}
			else
			{
				exporter.exportBaseFileMatrix(bas, file, parameters, reporterFields, spotFields, mergeReporters);
			}
			dc.commit();
			return file.getId();
		}
		catch (IOException e)
		{
			throw new BaseException(e);
		}
		catch (SQLException e)
		{
			throw new BaseException(e);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
	/**
		Loads a map from BASE 1 -> BASE 2 names of things. The 
		BASE 1 name must be the name of a formula or the external ID of an
		extra value in BASE 2 which should map to the corresponding value in BASE 2.
	*/
	private Map<String, String> loadNameMap(DbControl dc, BioAssaySet bas)
	{
		Map<String, String> names = new HashMap<String, String>();
		
		ItemQuery<Formula> queryFormula = Formula.getQuery(Formula.Type.COLUMN_EXPRESSION, 
			bas.getRawDataType());
		queryFormula.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
		queryFormula.order(Orders.asc(Hql.property("name")));
		for (Formula f : queryFormula.list(dc))
		{
			names.put(f.getName(), "frm."+f.getId());
		}
		
		names.put("lastUpdate", "rep.lastUpdate");
		names.put("geneName", "rep.name");
		names.put("position", "rep.position");
		names.put("reporter", "rep.id");
		names.put("reporterId", "rep.externalId");
		names.put("geneSymbol", "rep.symbol");
		
		names.put("rawPosition", "raw.position");
		names.put("block", "raw.block");
		names.put("numCol", "raw.col");
		names.put("numRow", "raw.row");
		names.put("x", "raw.x");
		names.put("y", "raw.y");
		
		for (ExtraValue ev : bas.getExtraValues().list(dc))
		{
			ExtraValueType evt = ev.getExtraValueType();
			names.put("_xc_"+evt.getExternalId(), "xtra."+evt.getExternalId());
		}
		
		return names;
	}
	
	/**
		Convert BASE 1 names to BASE 2 names. Names that can't be found in the name
		map are not converted. 
		@param base1Names An array of BASE 1 names
	*/
	private List<String> convertNames(Map<String, String> nameMap, String[] base1Names)
	{
		List<String> base2Names = new ArrayList<String>(base1Names.length);
		for (String name : base1Names)
		{
			if (nameMap.containsKey(name)) name = nameMap.get(name);
			base2Names.add(name);
		}
		return base2Names;
	}
	
	
	/**
		Get the experimental factors connected to a experiment.
		
		@return A list of {@link AnnotationType}:s
	*/
	private List<AnnotationType> getAnnotationTypes()
	{
		DbControl dc = null;
		List<AnnotationType> list = new ArrayList<AnnotationType>();
		try
		{
			dc = sc.newDbControl();
			Experiment e = super.getCurrentExperiment(dc);
			if (e != null)
			{
				ItemQuery<AnnotationType> query = e.getExperimentalFactors();
				query.include(Include.MINE, Include.SHARED, Include.IN_PROJECT, Include.OTHERS);
				query.order(Orders.asc(Hql.property("name")));
				list.addAll(query.list(dc));
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return list;
	}
	
	
	/**
		Get the data from the configuration file.
		
		@param f The file.
		@return A map of name->value pairs.
		@throws IOException If there i any problem reading from file.
	*/
	private Map<String, String> getParametersFromBaseFile(File f)
		throws IOException
	{
		FlatFileParser ffp = getInitializedFlatFileParser(f.getDownloadStream(0));
		ffp.setMinDataColumns(8);

		FlatFileParser.Line s = ffp.nextSection();
		ffp.parseHeaders();
		if (!s.name().equals("plugin"))
		{
			throw new BaseException("Can't find section 'plugin' in configuration file: " + f.getName());
		}
		
		HashMap<String, String> headers = new HashMap<String, String>();
		for (String header : ffp.getHeaderNames())
		{
			headers.put(header, ffp.getHeader(header));
		}
		
		Element parameters = new Element("jobparameters");
		while (ffp.hasMoreData())
		{
			FlatFileParser.Data dataline = ffp.nextData();
			try
			{
				JobParameter jp = new JobParameter(dataline.line().split("\t"));
				parameters.addContent(jp.toXml());
			}
			catch(Exception e)
			{
				throw new BaseException("Error on line "+dataline.lineNo()+" in the configuration file: " + f.getName(), e);
			}
		}
		XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
		headers.put(jobParametersParameter.getName(), out.outputString(parameters));
		return headers;
	}


	@SuppressWarnings({ "unchecked", "serial" })
	private TreeMap<String, JobParameter>
		getJobParameterObjectsFromXML(String xml, boolean hideRemoved)
	{
		try
		{
			String doctype = "<!DOCTYPE jobparameters SYSTEM \"base1-plugin-configuration-file.dtd\">";
			Document doc = XMLUtil.getValidatedXml(doctype+xml, dtdFile);
			Iterator<Element> it = doc.getDescendants(new Filter()
			{
				public boolean matches(Object o)
				{
					if (o instanceof Element)
					{
						Element e = (Element) o;
						return e.getName().equals("parameter");
					}
					return false;
				}
			});

			TreeMap<String, JobParameter> pars= new TreeMap<String, JobParameter>();
			while (it.hasNext())
			{
				Element e = it.next();
				JobParameter jp = new JobParameter(e);
				if (!jp.isRemoved())
				{
					if (pars.containsKey(jp.getName()))
					{
						throw new BaseException("Duplicate parameter at position "+jp.getPosition());
					}
					pars.put(jp.getName(), jp);
				}
			}
			return pars;
		}
		catch (JDOMException e1)
		{
			throw new BaseException(e1.getCause());
		}
		catch (IOException e1)
		{
			throw new BaseException(e1);
		}
	}


	@SuppressWarnings({ "unchecked", "serial" })
	private List<PluginParameter<?>> getJobParametersFromXML(String xml, boolean hideRemoved)
	{
		try
		{
			String doctype = "<!DOCTYPE jobparameters SYSTEM \"base1-plugin-configuration-file.dtd\">";
			Document doc = XMLUtil.getValidatedXml(doctype+xml, dtdFile);
			Iterator<Element> it = doc.getDescendants(new Filter()
			{
				public boolean matches(Object o)
				{
					if (o instanceof Element)
					{
						Element e = (Element) o;
						return e.getName().equals("parameter");
					}
					return false;
				}
			});
			
			TreeMap<Integer, PluginParameter<?>> positionParameter = new TreeMap<Integer, PluginParameter<?>>();
			while (it.hasNext())
			{
				Element e = it.next();
				JobParameter jp = new JobParameter(e);
				if (!jp.isRemoved())
				{
					if (positionParameter.containsKey(jp.getPosition()))
					{
						throw new BaseException("Duplicate parameter at position "+jp.getPosition());
					}
					positionParameter.put(jp.getPosition(), jp.getPluginParameter());
				}
			}
			return new ArrayList<PluginParameter<?>>(positionParameter.values());
		}
		catch (JDOMException e1)
		{
			throw new BaseException(e1.getCause());
		}
		catch (IOException e1)
		{
			throw new BaseException(e1);
		}
	}


	/**
		Creates a boolean from a string.
		
		@param s The string.
		@return false if s.equals("0") else true.
	*/
	private Boolean getBoolean(String s)
	{
		return s.equals("0") ? false : true;
	}
	
	/**
		Creates the information needed for manual configuration and initiates the 
		parameters. The parameters added are: versionNumber, usedColumns, usedFields, execName,
		geneAverages, SerialFormat, leaveStdin, leaveStdout, minChannel and maxCannel.
		
		@return The RequestInformation.
	*/
	private RequestInformation getManualConfigureParameters()
	{
		if (configureManual == null)
		{
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();

			versionNumberParameter = new PluginParameter<String>(
				"versionNumber",
				"Version",
				"The version of this plug-in",
				new StringParameterType(255, null, false));
			parameters.add(versionNumberParameter);
			
			usedColumnsParameter = new PluginParameter<String>(
				"usedColumns",
				"Columns",
				"A tab separeted sting of field namnes. A field is a value from the bioassay, the rawdata or feature",
				new StringParameterType(null, null, true));
			parameters.add(usedColumnsParameter);
			
			usedFieldsParameter = new PluginParameter<String>(
				"usedFields",
				"Fields",
				"A tab separeted sting of field namnes. A field is a value from the bioassay or the rawdata",
				new StringParameterType(null, null, true));
			parameters.add(usedFieldsParameter);
			
			execNameParameter = new PluginParameter<String>(
				"execName",
				"Name of executable",
				"The name of the file to be executed.",
				new StringParameterType(255, null, true));
			parameters.add(execNameParameter);
			
			geneAveragesParameter = new PluginParameter<Boolean>(
				"geneAverages",
				"Average on reporters",
				"Average on reporters within each assay",
				new BooleanParameterType(false, true));
			parameters.add(geneAveragesParameter);

			Enumeration<Boolean, String> format = new Enumeration<Boolean, String>();
			format.add(false, "Normal BASEfile (one big matrix)");
			format.add(true, "Serial BASEfile (one section per assay)");
			serialFormatParameter = new PluginParameter<Boolean>(
				"serialFormat",
				"Data format",
				"The format of the datafile can either be in a serial or matrix format",
				new BooleanParameterType(null, true, 1, format));
			parameters.add(serialFormatParameter);
			
			leaveStdinParameter = new PluginParameter<Boolean>(
				"leaveStdin",
				"Leave stdin.txt",
				"If the file stdin.txt should be removed after the plug-in execution or not",
				new BooleanParameterType(false, true));
			parameters.add(leaveStdinParameter);
			
			leaveStdoutParameter = new PluginParameter<Boolean>(
				"leaveStdout",
				"Leave stdout.txt",
				"If the file stdout.txt should be removed after the plug-in execution or not",
				new BooleanParameterType(false, true));
			parameters.add(leaveStdoutParameter);
			
			minChannelsParameter = new PluginParameter<Integer>(
				"minChannels",
				"Min channels",
				"Minimum number of channels",
				new IntegerParameterType(1, null, 2, true));
			parameters.add(minChannelsParameter);
			
			maxChannelsParameter = new PluginParameter<Integer>(
				"maxChannels",
				"Max channels",
				"Maximum number of channels. 0 for infinity",
				new IntegerParameterType(0, null, 0, true));
			parameters.add(maxChannelsParameter);
			
			parameters.add(jobParametersParameter);
			
			configureManual = new RequestInformation(
				Base1PluginExecuter.COMMAND_MANUAL_CONFIG,
				"Parse configuration file",
				"These parameters are parsed from the configuration file",
				parameters);
		}

		return configureManual;
	}

	
	/**
		Creates the information needed for job configuration and initiates the 
		parameters. The parameters added are: bioAssaySet and the parameters 
		found in the configuration file.
		
		@return The RequestInformation
	*/
	private RequestInformation getConfigureJobParameters()
	{
		if (configureJob == null)
		{
			// Parameters for CONFIGURE_JOB
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();
			parameters.add(getSourceBioAssaySetParameter("Bioassay set", "The bioassay set that this plugin will run on"));
			
			parameters.addAll(getJobParameters());
			configureJob = new RequestInformation(
				Request.COMMAND_CONFIGURE_JOB,
				"Configure job",
				"Set the parameters needed for this plugin.",
				parameters);
		}
		return configureJob;
	}
	
	
	/**
 		Creates the information needed for plugin configuration and initiates the 
		parameters. The parameters added are: file and execPath.
		
		@return The RequestInformation.
	*/
	private RequestInformation getConfigurePluginParameters()
	{
		if (configurePlugin == null)
		{
			// Parameters object for CONFIGURE_PLUGIN
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();
			parameters.add(fileParameter);
			parameters.add(execPathParameter);
			configurePlugin = new RequestInformation(
				Request.COMMAND_CONFIGURE_PLUGIN,
				"Configure plugin",
				"The configuration file from base1",
				parameters);
		}
		return configurePlugin;
	}

	
	/**
		Get the exec directory as a java.io.File. The exec directory is a sub directory in 
		"java.io.tmpdir".
		
		@return the directory.
	*/
	private java.io.File getExecDirectory()
	{
		if (execDirectory == null)
		{
			execDirectory = new java.io.File(
				System.getProperty("java.io.tmpdir") + java.io.File.separator + about.getName() + java.io.File.separator + job.getId());
			execDirectory.mkdirs();
		}
		return execDirectory;
	}
	
	
	private String getExecLine()
	{
		String execPath = (String) configuration.getValue(execPathParameter.getName());
		String execName = (String) configuration.getValue(execNameParameter.getName());
		String s = java.io.File.separator;
		if (execPath.endsWith(s) || execName.startsWith(s))
		{
			s = "";
		}
		String line = execPath + s + execName;
		return line;
	}

	
	/**
		Creates a {@link FlatFileParser} for a plugin configuration file.
		
		@param stream The stream that the parser should read from.
		@return The initialised parser
	*/
	private FlatFileParser getInitializedFlatFileParser(InputStream stream)
	{
		FlatFileParser ffp = new FlatFileParser();
		ffp.setSectionRegexp(Pattern.compile("section[\\t ](.*)"));
		ffp.setHeaderRegexp(Pattern.compile("(.*?)\\t(.*)"));
		ffp.setDataHeaderRegexp(Pattern.compile("%"));
		ffp.setDataFooterRegexp(Pattern.compile("^$"));
		ffp.setDataSplitterRegexp(Pattern.compile("\\t"));
		ffp.setUseNullIfEmpty(false);

		ffp.setInputStream(stream, Config.getCharset());
		return ffp;
	}
	

	/**
		Reads the configuration file an creates the parameters found there.
		
		@return A list of {@link PluginParameter}:s
	*/
	private List<PluginParameter<?>> getJobParameters()
		throws BaseException
	{
		if (jobParameters != null)
		{
			return jobParameters;
		}
		jobParameters = new ArrayList<PluginParameter<?>>();
				
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();
			Experiment experiment = getCurrentExperiment(dc);
			String homeDirectoryPath = "/" + Path.makeSafeFilename(PluginConfiguration.getById(dc, configuration.getId()).getName(), "");
			Directory homeDirectory = experiment == null ? null : experiment.getDirectory();
			if (homeDirectory != null)
			{
				homeDirectoryPath = homeDirectory.getPath() + homeDirectoryPath;
			}
			else
			{
				homeDirectory = User.getById(dc, sc.getLoggedInUserId()).getHomeDirectory();
				if (homeDirectory != null)
				{
					homeDirectoryPath = homeDirectory.getPath() + homeDirectoryPath;
				}
			}
			pluginDirectoryParameter = new PluginParameter<String>(
				"pluginDirectory",
				"Plugin directory",
				"The directory where a plugin can store its files.",
				new PathParameterType(Path.Type.DIRECTORY, homeDirectoryPath, true));
			jobParameters.add(pluginDirectoryParameter);
			
			Object xml = configuration.getValue(jobParametersParameter.getName());
			// To make the plugin compatible with older configurations
			if (xml == null)
			{
				throw new BaseException("Cant find the jobparameters specification. Reconfigure the plugin.");
			}
			jobParameters.addAll(getJobParametersFromXML(xml.toString(), true));
		}
		finally
		{
			if (dc != null) dc.close();
		}

		return jobParameters;
	}
	
	/**
		Get a {@link Directory directory} 
	*/
	private Directory getPluginDirectory(DbControl dc)
	{
		if (pluginDirectoryId == -1)
		{
			pluginDirectoryId = createPluginDirectory();
		}
		return Directory.getById(dc, pluginDirectoryId);
	}
	
	
	/**
		Imports the data from the file stdout.txt in the execution directory.
		@throws IOException if there is any error reading from stdout.txt
	*/
	private void importData(DbControl dc, File stdout, Transformation t)
		throws SQLException, IOException
	{
		
		// Position -> ReporterID mapping for the parent bioassay set
		Map<Integer, Integer> parentReporterPositions = new HashMap<Integer, Integer>();
		// Position -> ReporterID mapping for the child bioassay set
		Map<Integer, Integer> childReporterPositions = new HashMap<Integer, Integer>();
		// BioAssayID -> Column no mapping for the child bioassay set
		Map<Integer, Short> bioAssayColumns = new HashMap<Integer, Short>();
		// Column mapping from parent -> child bioassay
		Map<Short, Short> columnMapping = new HashMap<Short, Short>();
		// If the "assays" section contains a child->parent mapping for bioassays
		boolean hasParentAssayMapping = false;
		// If all position->reporter mappings are the same on the child and parent bioassay set
		boolean hasSamePositionReporterMapping = true;
		// If at least one spot section was found
		boolean hasSpotSection = false;
		// Information about child bioassays
		Map<Integer, ChildBioAssay> childBioAssays = new HashMap<Integer, ChildBioAssay>();
		
		BioAssaySet source = t.getSource();
		int numSections = 0;
		// Parse the stdout file to verify child mappings
		FlatFileParser ffp = getInitializedFlatFileParser(stdout.getDownloadStream(0));
		while (ffp.hasMoreSections())
		{
			checkInterrupted();
			numSections++;
			FlatFileParser.Line section = ffp.nextSection();
			ffp.parseHeaders();
			
			if ("assays".equals(section.name()))
			{
				/*
					Optional section
					If it exists it must define 'id' and 'name' of child bioassays
					The 'id' must be the same as the id of a parent bioassay,
					UNLESS a 'parents' column is present
				*/
				List<String> columns = Arrays.asList(ffp.getHeader("columns").split("\\t"));
				int parentIndex = columns.indexOf("parents");
				int idIndex = columns.indexOf("id");
				int nameIndex = columns.indexOf("name");
				hasParentAssayMapping =  parentIndex != -1;
				if (idIndex == -1)
				{
					throw new BaseException("Missing column 'id' in section 'assays' at line " 
							+ ffp.getParsedLines() + " in file '" + stdout.getName() + "'");
				}
				if (nameIndex == -1)
				{
					throw new BaseException("Missing column 'name' in section 'assays' at line " 
							+ ffp.getParsedLines() + " in file '" + stdout.getName() + "'");	
				}
				
				// Read data part of the 'assays' section
				// Each line has ID, NAME and PARENTS (optional)
				ffp.setMinDataColumns(columns.size());
				FlatFileParser.Data data;
				while ((data = ffp.nextData()) != null)
				{
					int assayId = Values.getInt(data.get(idIndex));
					
					// Don't allow duplicate assay ID:s
					if (childBioAssays.containsKey(assayId))
					{
						throw new BaseException("Duplicate assay id (" + assayId + 
								") found in section 'assays' at line " + ffp.getParsedLines() + 
								" in file '" + stdout.getName() + "'");
					}
					
					// Extract name and parents and store as ChildBioAssay objects
					String name = data.get(nameIndex);
					Set<Integer> parents = null;
					if (hasParentAssayMapping)
					{
						parents = new HashSet<Integer>(Arrays.asList(Values.getInt(data.get(parentIndex).split("/"))));
						Integer parentId = null;						
						try
						{
							Iterator<Integer> it = parents.iterator();
							while (it.hasNext())
							{
								parentId = it.next();
								BioAssay.getById(dc, parentId);
							}
						}
						catch (ItemNotFoundException ex)
						{
							throw new BaseException("Can't find parent bioassay '" + parentId +
								"' in section 'assays' at line " + ffp.getParsedLines() + " in file '" +
								stdout.getName() + "'"
								);
						}
					}
					else
					{
						try
						{
							BioAssay.getById(dc, assayId);
						}
						catch (ItemNotFoundException ex)
						{
							throw new BaseException("Can't find bioassay '" + assayId +
								"' in section 'assays' at line " + ffp.getParsedLines() + " in file '" +
								stdout.getName() + "'"
								);
						}
					}
					childBioAssays.put(assayId, new ChildBioAssay(assayId, name, parents));
				}
			}
			else if ("spots".equals(section.name()))
			{
				checkInterrupted();
				/*
					Optional section. May appear more than once.
					'position' and 'reporter' are required columns in the 'columns' header.
					'assays' is a required column header, that has the ID of the assays
					that have spot data in this section.
				*/
				if (!hasSpotSection)
				{
					hasSpotSection = true;
					// Load the position -> reporter mapping from the parent bioassay set
					DynamicQuery positionQuery = source.getPositionData();
					positionQuery.select(Dynamic.select(VirtualColumn.POSITION));
					positionQuery.select(Dynamic.select(VirtualColumn.REPORTER_ID));
					DynamicResultIterator positionQueryIterator = positionQuery.iterate(dc);
					while (positionQueryIterator.hasNext())
					{
						SqlResult result = positionQueryIterator.next();
						parentReporterPositions.put(result.getInt(1), (Integer)result.getObject(2));
					}
				}

				// Get the "position" and "reporter" columns from the "column" header
				List<String> columns = Arrays.asList(ffp.getHeader("columns").split("\\t"));
				int positionIndex = columns.indexOf("position");
				int reporterIndex = columns.indexOf("reporter");

				// Error if the columns are missing
				if (positionIndex == -1) 
				{
					throw new BaseException("Missing column 'position' in section 'spots' at line " +
							ffp.getParsedLines() + " in file '" + stdout.getName() + "'");
				}
				if (reporterIndex == -1) 
				{
					throw new BaseException("Missing column 'reporter' in section 'spots' at line " +
							ffp.getParsedLines() + " in file '" + stdout.getName() + "'");
				}

				// Get the "assays" header
				List<Integer> assays = Arrays.asList(Values.getInt(ffp.getHeader("assays").split("\\t")));
				if (assays == null || assays.size() == 0)
				{
					throw new BaseException("Missing header 'assays' in section 'spots' at line " +
							ffp.getParsedLines() + " in file '" + stdout.getName() + "'");
				}
				
				for (Integer assayId : assays)
				{
					if (!childBioAssays.containsKey(assayId))
					{
						try
						{
							BioAssay.getById(dc, assayId);
						}
						catch (ItemNotFoundException ex)
						{
							throw new BaseException("Can't find bioassay '" + assayId +
								"' in section 'spots' at line " + ffp.getParsedLines() + " in file '" +
								stdout.getName() + "'"
								);
						}
						childBioAssays.put(assayId, new ChildBioAssay(assayId, null, null));
					}
				}
				
				// Parse data and check if each position has same reporter as before
				FlatFileParser.Data data;
				while ((data = ffp.nextData()) != null)
				{
					checkInterrupted();
					Integer position = Values.getInteger(data.get(positionIndex), null);
					Integer newReporterId = Values.getInteger(data.get(reporterIndex), null);
					
					// Error if position is missing
					if (position == null)
					{
						throw new BaseException("Missing or invalid value for 'position' in section 'spots' at line " +
								ffp.getParsedLines() + " in file '" + stdout.getName() + "'");
					}
					
					// Is the new reporter the same as the existing?
					// A 'null' reporter is considered the same as another 'null' reporter
					if (hasSamePositionReporterMapping)
					{			
						Integer existingReporterId = parentReporterPositions.get(position);
						if (!Diff3.isEqualOrNull(newReporterId, existingReporterId))
						{
							hasSamePositionReporterMapping = false;
							parentReporterPositions.clear();
						}
					}
					
					// Is the new reporter already registered with the same position?
					if (!childReporterPositions.containsKey(position))
					{
						childReporterPositions.put(position, newReporterId);
					}
					else
					{
						Integer registeredReporterId = childReporterPositions.get(position);
						// Error if same position has different reporters in the stdout file
						if (!Diff3.isEqualOrNull(newReporterId, registeredReporterId))
						{
							throw new BaseException("Invalid value for 'reporter' (" + newReporterId + 
								") for position '" + position + "' in section 'spots' at line " +	
								ffp.getParsedLines() + " in file '" + stdout.getName() + 
								"'. Expected '" + registeredReporterId + "'");
						}
					}
				}
			}
			else if ("reporterlist".equals(section.name()))
			{
				// Reporter lists are imported immediately
				// Get the "reporterId" and "score" columns from the "column" header
				List<String> columns = Arrays.asList(ffp.getHeader("columns").split("\\t"));
				int reporterIndex = columns.indexOf("reporterId");
				int scoreIndex = columns.indexOf("score");

				// Error if the reporterId column is missing
				if (reporterIndex == -1) 
				{
					throw new BaseException("Missing column 'reporterId' in section 'reporterlist' at line " +
							ffp.getParsedLines() + " in file '" + stdout.getName() + "'");
				}
				
				String name = Values.getStringOrNull(ffp.getHeader("name"));
				if (name == null) name = t.getName() + "." + numSections;
				ReporterList list = ReporterList.getNew(dc);
				list.setName(name);
				list.setExperiment(t.getExperiment());
				
				FlatFileParser.Data data;
				ReporterBatcher reporters = ReporterBatcher.getNew(dc);
				while ((data = ffp.nextData()) != null)
				{
					checkInterrupted();
					String reporterId = Values.getStringOrNull(data.get(reporterIndex));
					
					// Ignore missing reporters
					if (reporterId == null) continue;
					
					ReporterData reporter = reporters.getByExternalId(reporterId, false);
					if (reporter == null)
					{
						throw new ItemNotFoundException("Reporter[externalId=" + reporterId + "] at line "+
								ffp.getParsedLines() + " in file '" + stdout.getName() + "'");
					}
					// If no score is provided we use the line number as some kind of 'rank'
					Float score = scoreIndex == -1 ? 
						data.dataLineNo() : Values.getFloat(data.get(scoreIndex), null);
					ReporterProxy proxy = new ReporterProxy();
					list.addReporter(reporter, score);
				}

				dc.saveItem(list);
				AnyToAny any2Any = AnyToAny.getNew(dc, t, list, "reporterList." + numSections, false);
				dc.saveItem(any2Any);
			}
		}
		
		if (!hasSpotSection) return;
		
		// Create the child bioassay set
		BioAssaySet child = null;
		boolean useNewDataCube = hasParentAssayMapping || !hasSamePositionReporterMapping;
		if (useNewDataCube)
		{
			child = t.newProduct("new", "new", false);
		}
		else
		{
			child = t.newProduct(null, "new", false);
		}
		dc.saveItem(child);

		// Create child bioassays
		for (ChildBioAssay childData : childBioAssays.values())
		{
			BioAssay childBa = null;
			if (useNewDataCube)
			{
				Collection<BioAssay> parents = null;
				if (childData.parents == null)
				{
					parents = Collections.singletonList(BioAssay.getById(dc, childData.id));
				}
				else
				{
					parents = new ArrayList<BioAssay>(childData.parents.size());
					for (Integer parentId : childData.parents)
					{
						parents.add(BioAssay.getById(dc, parentId));
					}
				}
				childBa = child.newBioAssay(parents);
				// Put column mappings
				short childColumn = childBa.getDataCubeColumnNo();
				for (BioAssay parentBa : parents)
				{
					columnMapping.put(parentBa.getDataCubeColumnNo(), childColumn);
				}
			}
			else
			{
				BioAssay parentBa = BioAssay.getById(dc, childData.id);
				childBa = child.newBioAssay(parentBa);
				// Columns should be identical... put in the mapping just be be sure
				columnMapping.put(parentBa.getDataCubeColumnNo(), childBa.getDataCubeColumnNo());
			}
			if (childData.name != null) childBa.setName(childData.name);
			dc.saveItem(childBa);
			bioAssayColumns.put(childData.id, childBa.getDataCubeColumnNo());
		}
		
		// Create new position -> reporter mapping
		if (useNewDataCube)
		{
			// Create position -> reporter mapping from the new data
			PositionBatcher posBatcher = child.getPositionBatcher();
			ReporterProxy proxy = new ReporterProxy();
			for (Map.Entry<Integer, Integer> entry : childReporterPositions.entrySet())
			{
				checkInterrupted();
				int position = entry.getKey();
				Integer reporterId = entry.getValue();
				if (reporterId == null)
				{
					posBatcher.insert(position, null);
				}
				else
				{
					proxy.setTheId(reporterId);
					posBatcher.insert(position, proxy);
				}
			}
			posBatcher.flush();
			posBatcher.close();
			
			if (hasSamePositionReporterMapping)
			{
				// Create raw data mapping
				MappingBatcher mapBatcher = child.getMappingBatcher();
				DynamicSpotQuery spotQuery = source.getSpotData();
				spotQuery.joinRawData(JoinType.INNER);
				
				spotQuery.select(Dynamic.select(VirtualColumn.COLUMN));
				spotQuery.select(Dynamic.select(VirtualColumn.POSITION));
				spotQuery.select(Dynamic.select(VirtualTable.RAWPARENTS, VirtualColumn.RAWDATA_ID));
				
				DynamicResultIterator spotIterator = spotQuery.iterate(dc);
				RawDataProxy rawProxy = new RawDataProxy();
				while (spotIterator.hasNext())
				{
					SqlResult result = spotIterator.next();
					short parentColumn = result.getShort(1);
					int position = result.getInt(2);
					int rawDataId = result.getInt(3);
					Short childColumn = columnMapping.get(parentColumn);
					if (childColumn != null)
					{
						rawProxy.setTheId(rawDataId);
						mapBatcher.insert(childColumn, position, rawProxy);
					}
				}
				mapBatcher.flush();
				mapBatcher.close();
			}
		}
		
		// Clean up things that are no longer needed
		childBioAssays.clear();
		childReporterPositions.clear();
		parentReporterPositions.clear();
		
		
		// Old code below

		SpotBatcher spotBatcher = child.getSpotBatcher();
		HashMap<String, ExtraValueType> evtMap = new HashMap<String, ExtraValueType>();
		ffp = getInitializedFlatFileParser(stdout.getDownloadStream(0));
		
		while (ffp.hasMoreSections())
		{
			checkInterrupted();
			FlatFileParser.Line section = ffp.nextSection();
			ffp.parseHeaders();
			
			if (section.name().equals("assays"))
			{
				List<String> columns = Arrays.asList(ffp.getHeader("columns").split("\\t"));
				ffp.setMinDataColumns(columns.size());
				// Loop to the end of this section
				while (ffp.nextData() != null)
				{}
			}
			else if (section.name().equals("spots"))
			{
				List<String> columns = Arrays.asList(ffp.getHeader("columns").split("\\t"));
				List<Integer> assays = Arrays.asList(Values.getInt(ffp.getHeader("assays").split("\\t")));
				List<String> assayFields = Arrays.asList(ffp.getHeader("assayFields").split("\\t"));
				List<String> setExtraFloats = new ArrayList<String>(); 
				if (ffp.getHeader("setExtraFloats") != null)
				{
					setExtraFloats = Arrays.asList(ffp.getHeader("setExtraFloats").split("\\t"));
				}
				ffp.setMinDataColumns(columns.size() - 1 + assays.size() * assayFields.size());
				
				boolean intCols = true;

				int posCol = columns.indexOf("position");
				int dataCol = columns.indexOf("assayData");
				int[] intCol = new int[child.getRawDataType().getChannels()];
				for (int i = 0; i < intCol.length; ++i)
				{
					int col = assayFields.indexOf("intensity"+(i+1));
					if (col != -1)
					{
						intCol[i] = col;
					}
					else
					{
						intCols = false;
					}
				}
				int mCol = assayFields.indexOf("l2ratio1_2");
				int aCol = assayFields.indexOf("l10intgmean1_2");
				int[] extraFloatsCol = new int[setExtraFloats.size()];

				if (dataCol > -1)
				{
					if (!intCols && (mCol > -1 && aCol > -1))
					{
						intCols = false;
					}
					else if (!intCols)
					{
						throw new BaseException(
							"Can't find the intensity column(s) or l2ratio1_2/l10intgmean1_2. No data could be imported.");
					}
					for (int i = 0; i < setExtraFloats.size(); i++)
					{
						int col = assayFields.indexOf(setExtraFloats.get(i));
						if (col < 0)
						{
							throw new BaseException(
								"Can't find the column " + setExtraFloats.get(i) + ". No data could be inmported.");
						}
						extraFloatsCol[i] = col + dataCol;
					}
				}
				else
				{
					throw new BaseException("Can't find the column 'assayData' in header assayFields.");
				}

				for (int i = 0; i < intCol.length; ++i)
				{
					intCol[i] += dataCol;
				}
				aCol += dataCol;
				mCol += dataCol;
				
				List<SpotExtraValueBatcher<Float>> evBatcher = new ArrayList<SpotExtraValueBatcher<Float>>(setExtraFloats.size());
				for (int i = 0; i < setExtraFloats.size(); i++)
				{
					String name = setExtraFloats.get(i);
					ExtraValueType evt = evtMap.get(name);
					if (evt == null)
					{
						try
						{
							evt = ExtraValueType.getByExternalId(dc, name);
						}
						catch (ItemNotFoundException e)
						{
							evt = ExtraValueType.getNew(dc, name, Type.FLOAT);
							evt.setName(name);
							dc.saveItem(evt);
						}
						evtMap.put(name, evt);
					}
					evBatcher.add(i, child.getSpotExtraValueBatcher(Float.class, evt, Job.getById(dc, job.getId())));
				}
				
				float[] intensities = new float[intCol.length];
				while (ffp.hasMoreData())
				{
					checkInterrupted();
					FlatFileParser.Data dataline = ffp.nextData();
					int index = 0;
					Integer position = Values.getInteger(dataline.get(posCol), null);
					for (Integer assayId : assays)
					{
						// Only insert the spot if all intensity values are correct numbers
						boolean insertSpot = true;
						short dataCubeColumn = bioAssayColumns.get(assayId);
						if (intCols)
						{
							for (int i = 0; i < intCol.length; ++i)
							{
								float value = Values.getFloat(dataline.get(intCol[i] + index), Float.NaN);
								insertSpot &= !Float.isNaN(value);
								intensities[i] = value;
							}
						}
						else
						{
							float a = Values.getFloat(dataline.get(aCol + index), Float.NaN);
							float m = Values.getFloat(dataline.get(mCol + index), Float.NaN);
							insertSpot = !Float.isNaN(m) && !Float.isNaN(a);
							if (insertSpot)
							{
								// int2 = 10^a / 2^(0.5*m)
								// int1 = int2 * 2^m
								intensities[1] = (float)(Math.pow(10, a) / Math.pow(2, 0.5 * m));
								intensities[0] = (float)(intensities[1] * Math.pow(2, m));
							}
						}
						if (insertSpot) 
						{
							spotBatcher.insert(dataCubeColumn, position, intensities);
							// Extra values
							for (int i = 0; i < evBatcher.size(); i++)
							{
								float value = Values.getFloat(dataline.get(extraFloatsCol[i] + index), Float.NaN);
								if (!Float.isNaN(value)) 
								{
									evBatcher.get(i).insert(dataCubeColumn, position, value);
								}
							}
						}
						index += assayFields.size();
					}
				}
			}
		}
	}
	
	private void importFiles(DbControl dc, Transformation trans)
	{
		DbControl mydc = null;
		try
		{
			mydc = sc.newDbControl();
			Directory homeDirectory = getPluginDirectory(mydc);
			importTempFiles(mydc, getExecDirectory().listFiles(), homeDirectory);
			mydc.commit();
		}
		finally
		{
			if (mydc != null)
				mydc.close();
		}
		
		checkInterrupted();
		Directory homeDirectory = getPluginDirectory(dc);
		for(File f : homeDirectory.getFiles().list(dc))
		{
			AnyToAny ata = AnyToAny.getNew(dc, trans, f, f.getPath().toString(), false);
			dc.saveItem(ata);
		}
		for(Directory d : homeDirectory.getSubDirectories().list(dc))
		{
			AnyToAny ata = AnyToAny.getNew(dc, trans, d, d.getPath().toString(), false);
			dc.saveItem(ata);
		}
	}


	private void importTempFiles(DbControl dc, java.io.File[] files, Directory d)
	{
		for (java.io.File f : files)
		{
			if (f.isFile())
			{
				if (!File.exists(dc, d, f.getName()))
				{
					File newFile = File.getNew(dc, d);
					newFile.setName(f.getName());
					newFile.setMimeTypeAuto(null, null);
					dc.saveItem(newFile);
					try
					{
						checkInterrupted();
						newFile.upload(new FileInputStream(f), true);
					}
					catch (FileNotFoundException e)
					{
						continue;
					}
				}
			}
			else if (f.isDirectory())
			{
				Directory newDirectory = d.newSubDirectory();
				newDirectory.setName(f.getName());
				dc.saveItem(newDirectory);
				importTempFiles(dc, f.listFiles(), newDirectory);
			}
		}
	}
	
	
	/**
		This thread read everything from one input stream ant writes everything to an output stream.
		
		@author Johan Enell
	*/
	private class StreamHandler 
		extends Thread
	{
		/**
			The input stream to read from;
		*/
		private InputStream is;

		/**
			The output stream to write to.
		*/
		private OutputStream os;

		/**
			Creates a steam handler connected to the specific streams.
			
			@param is The stream to read from.
			@param redirect The stream to write to.
		*/
		StreamHandler(InputStream is, OutputStream redirect)
		{
			this.is = is;
			this.os = redirect;
		}

		@Override
		public void run()
		{
			try
			{
		    byte[] buf = new byte[1024];
		    int i = 0;
		    while((i = is.read(buf)) != -1)
		    {
		    	os.write(buf, 0, i);
		    }
		    is.close();
		    os.close();
			}
			catch (IOException e)
			{}
		}
	}

	
	private class JobParameter
	{
		private int position;
		
		private Base1JobParameterType type;
		
		private String name;
		
		private String commonName;
		
		private String options;
		
		private String defaultValue;
		
		private Enumeration<String, String> enumOptions = new Enumeration<String, String>();
		
		private boolean removed = false;

		JobParameter(String... base1ConfigLine)
		{
			if (base1ConfigLine.length != 8)
			{
				throw new BaseException("Must have 8 parameters.");
			}
			if (base1ConfigLine[0].equals(""))
			{
				throw new BaseException("Position cannot be null");
			}
			position = Integer.parseInt(base1ConfigLine[0]);
			if (base1ConfigLine[1].length() != 1)
			{
				throw new BaseException("Type must one character");
			}
			type = Base1JobParameterType.get(base1ConfigLine[1].charAt(0));
			name = base1ConfigLine[2];
			commonName = base1ConfigLine[3];
			options = base1ConfigLine[4];
			defaultValue = base1ConfigLine[5];
			
			if (type == Base1JobParameterType.ENUMERATION)
			{
				String[] enums = base1ConfigLine[6].split("\\\\t");
				if (enums.length % 2 != 0) throw new BaseException("name/value pairs in enum isnt even: "+base1ConfigLine[6]);
				for (int i = 0; i < enums.length; i += 2)
				{
					enumOptions.add(enums[i], enums[i+1]);
				}
			}
			removed = Values.getBoolean(base1ConfigLine[7]);
		}

		@SuppressWarnings({ "unchecked", "serial" })
		JobParameter(Element e)
		{
			try
			{
				position = e.getAttribute("position").getIntValue();
				type = Base1JobParameterType.get(e.getAttribute("type").getValue().charAt(0));
				Iterator<Element> it = e.getDescendants(new Filter()
				{
					public boolean matches(Object o)
					{
						if (o instanceof Element)
						{
							return true;
						}
						return false;
					}
				});
				while (it.hasNext())
				{
					Element child = it.next();
					if (child.getName().equals("name"))
					{
						name = child.getValue();
					}
					else if (child.getName().equals("commonname"))
					{
						commonName = child.getValue();
					}
					else if (child.getName().equals("options"))
					{
						options = child.getValue();
					}
					else if (child.getName().equals("defaultvalue"))
					{
						defaultValue = child.getValue();
					}
					else if (child.getName().equals("removed"))
					{
						removed = true;
					}
					else if (child.getName().equals("enumoptions"))
					{
						Iterator<Element> it2 = child.getDescendants(new Filter()
						{

							public boolean matches(Object o)
							{
								if (o instanceof Element)
								{
									return true;
								}
								return false;
							}
						});
						while (it2.hasNext())
						{
							Element value = it2.next();
							enumOptions.add(value.getAttributeValue("key"), value.getValue());
						}
					}
				}
			}
			catch (JDOMException ex)
			{
				throw new BaseException(ex);
			}
		}

		public PluginParameter<?> getPluginParameter()
		{
			PluginParameter<?> parameter;
			switch (type)
			{
				case STRING:
				{
					StringParameterType t = new StringParameterType(65535, defaultValue, false, 1, new Integer(options), 1);
					parameter = new PluginParameter<String>(name, commonName, "", t);
					break;
				}
				case INTEGER:
				{
					Integer defValue = Values.getInteger(defaultValue, null);
					IntegerParameterType t = new IntegerParameterType(
						null,
						null,
						defValue,
						false,
						1,
						Values.getInt(options, 30),
						1,
						null);
					parameter = new PluginParameter<Integer>(name, commonName, "", t);
					break;
				}
				case HIDDEN_STRING:
				{
					StringParameterType t = new StringParameterType(65535, defaultValue, false, 1, 0, 1);
					parameter = new PluginParameter<String>(name, null, "", t);
					break;
				}
				case FLOAT:
				{
					Float defValue = Values.getFloat(defaultValue, null);
					FloatParameterType t = new FloatParameterType(null, null, defValue, false, 1, Values.getInt(options, 30), 1, null);
					parameter = new PluginParameter<Float>(name, commonName, "", t);
					break;
				}
				case TEXT:
				{
					String[] wh = options.split(",");
					int width = Values.getInt(wh[0], 30);
					int height = wh.length > 1 ? Values.getInt(wh[1], 1) : 1;
					StringParameterType t = new StringParameterType(null, defaultValue, false, 1, width, height);
					parameter = new PluginParameter<String>(name, commonName, "", t);
					break;
				}
				case ANNOTATION:
				{
					List<AnnotationType> annotations = getAnnotationTypes();
					Enumeration<String, String> enums = new Enumeration<String, String>();
					for (AnnotationType at : annotations)
					{
						enums.add(at.getName(), at.getName());
					}
					StringParameterType t = new StringParameterType(65535, null, false, 1, Values.getInt(options, 30), 1, enums);
					parameter = new PluginParameter<String>(name, commonName, "", t);
					break;
				}
				case ENUMERATION:
				{
					StringParameterType t = new StringParameterType(65535, defaultValue, false, 1, Values.getInt(options, 30), 1, enumOptions);
					parameter = new PluginParameter<String>(name, commonName, "", t);
					break;
				}
				default:
					parameter = null;
			}
			return parameter;
		}
		
		public final String getCommonName()
		{
			return commonName;
		}

		public final String getDefaultValue()
		{
			return defaultValue;
		}

		public final Enumeration<String, String> getEnumOptions()
		{
			return enumOptions;
		}

		public final String getName()
		{
			return name;
		}

		public final String getOptions()
		{
			return options;
		}

		public final int getPosition()
		{
			return position;
		}

		public final boolean isRemoved()
		{
			return removed;
		}

		public final Base1JobParameterType getType()
		{
			return type;
		}

		public Element toXml()
		{
			Element parameter = new Element("parameter");
			parameter.setAttribute("position", String.valueOf(getPosition()));
			parameter.setAttribute("type", getType().toString());
			
			parameter.addContent(new Element("name").setText(getName()));
			parameter.addContent(new Element("commonname").setText(getCommonName()));
			parameter.addContent(new Element("options").setText(getOptions()));
			parameter.addContent(new Element("defaultvalue").setText(getDefaultValue()));
			
			if (enumOptions.size() > 0)
			{
				Element enums = new Element("enumoptions");
				for (int i = 0; i < enumOptions.size(); ++i)
				{
					Element value = new Element("value");
					value.setAttribute("key", enumOptions.getKey(i));
					value.setText(enumOptions.getValue(i));
					enums.addContent(value);
				}
				parameter.addContent(enums);
			}
			if (removed) parameter.addContent(new Element("removed"));
			return parameter;
		}

		@Override
		public String toString()
		{
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			return out.outputString(toXml());
		}
	}
	
	private class ListMap<K, V> extends java.util.AbstractMap<K, V>
	{
		private List<K> keys = new ArrayList<K>();
		private List<V> values = new ArrayList<V>();

		@Override
		public Set<java.util.Map.Entry<K, V>> entrySet()
		{
			throw new UnsupportedOperationException();
		}

		@Override
		public V put(K key, V value)
		{
			V ret = null;
			int index = keys.indexOf(key);
			if (index == -1)
			{
				keys.add(key);
				values.add(value);
			}
			else
			{
				keys.set(index, key);
				ret = values.set(index, value);
			}
			return ret;
		}

		@Override
		public V get(Object key)
		{
			if (key == null) throw new NullPointerException();
			int index = keys.indexOf(key);
			V ret = null;
			if (index != -1)
			{
				ret = values.get(index);
			}
			return ret;
		}

		@Override
		public boolean isEmpty()
		{
			return keys.isEmpty();
		}

		@Override
		public Set<K> keySet()
		{
			return new AbstractSet<K>()
			{
				@Override
				public Iterator<K> iterator()
				{
					return Collections.unmodifiableList(keys).iterator();
				}

				@Override
				public int size()
				{
					return keys.size();
				}
			};
		}
		
	}

	private enum Base1JobParameterType
	{
		HIDDEN_STRING('h'), STRING('t'), INTEGER('i'), FLOAT('f'), TEXT('a'), ANNOTATION('n'), ENUMERATION('e');
		
		private final char c;
		
		private Base1JobParameterType(char c)
		{
			this.c = c;
		}
		
		public static Base1JobParameterType get(char c)
		{
			switch (c)
			{
				case 'h' : return HIDDEN_STRING;
				case 't' : return STRING;
				case 'i' : return INTEGER;
				case 'f' : return FLOAT;
				case 'a' : return TEXT;
				case 'n' : return ANNOTATION;
				case 'e' : return ENUMERATION;
				default:
					throw new BaseException("Unknown type: "+c);
			}
		}

		@Override
		public String toString()
		{
			return String.valueOf(c);
		}
	}
	
	private static class ReporterProxy
		extends ReporterData
	{
		private int id;
		ReporterProxy()
		{}
		public void setTheId(int id)
		{
			this.id = id;
		}
		public int getId()
		{
			return id;
		}
	}
	
	private static class RawDataProxy
		extends RawData
	{
		private int id;
		RawDataProxy()
		{}
		public void setTheId(int id)
		{
			this.id = id;
		}
		public int getId()
		{
			return id;
		}
	}	
	
	private static class ChildBioAssay
	{
		final int id;
		final String name;
		final Set<Integer> parents;
		
		ChildBioAssay(int id, String name, Set<Integer> parents)
		{
			this.id = id;
			this.name = name;
			this.parents = parents;
		}
	}
}
