/**
	$Id: IdMethod.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.plugins.batchimport;

import java.util.List;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;

/**
	Represents an 'identification method', eg. a way to find an item among
	the items that already exists in the database. This can, for example, be done
	by checking the name or the internal id of the item.

	@author nicklas
	@version 2.8
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
 */
public interface IdMethod
{
	/**
		The name of this method, which should be unique among the methods that
		can be used at a given time.
		@return The method name
	*/
	public String getMethod();

	/**
		The display title of the method.
	*/
	public String getTitle();
	
	/**
		If the identification method is truly unique or not. If this method
		returns true, the {@link #find(DbControl, ItemQuery, String)} method will 
		never ever return a list with more than one item.
	*/
	public boolean isTrulyUnique();

	public String getColumnMappingParameterName();
	
	/**
		Initialise a query to be used by this identification method. The
		query should be a fresh query without any previous restrictions or
		other modifications.
		@param query The query to prepare
		@return The query that is used for item lookup
	*/
	public <I extends BasicItem> ItemQuery<I> prepareQuery(DbControl dc, ItemQuery<I> query);
	
	/**
		Try to find items with this query method.
		
		@param query The query to use, which should have been prepared
			using {@link #prepareQuery(DbControl, ItemQuery)}
		@param identifier The identifier of the item
		@return A list with the found items, if {@link #isTrulyUnique()} return true
			the list is guaranteed to have at most one item
	*/
	public <I extends BasicItem> List<I> find(DbControl dc, ItemQuery<I> query, String identifier);

}
