/*
	$Id: PermissionPlugin.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson
 
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

 	You should have received a copy of the GNU General Public License
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330,
 	Boston, MA  02111-1307, USA.
*/

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.News;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.AbstractPlugin;
import net.sf.basedb.core.plugin.Plugin;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

public class PermissionPlugin 
	extends AbstractPlugin
{

	public MainType getMainType()
	{
		return Plugin.MainType.OTHER;
	}
	
	public About getAbout()
	{		
		return new AboutImpl
		(
				"Permission plugin",
				"This plugin is testing the plugin permissions.",
				"2.0",
				"2006, Base 2 development team",
				null,
				null,
				"http://base.thep.lu.se"
			);
	}

	public boolean supportsConfigurations()
	{
		return false;
	}

	public boolean requiresConfiguration()
	{	
		return false;
	}

	public void run(Request request, Response response, ProgressReporter progress)
		throws BaseException
	{	
		String cmd = (String)job.getValue("cmd");
		boolean shouldFail = (Boolean)job.getValue("shouldFail");

		try
		{
			if ("create".equals(cmd))
			{
				try
				{
					createNews(shouldFail ? 
						"This news should not be here because we expected a PermissionDeniedException" : 
						"This news should be here since we now have CREATE permission");
				}
				catch (PermissionDeniedException ex)
				{
					if (shouldFail)
					{
						response.setDone("Expected failure for command " + cmd + ": " + ex.getMessage());
					}
					else
					{
						response.setError("Unexpected failure for command " + cmd + ": " + ex.getMessage(), Arrays.asList(ex));			 
					}
					return;
				}
			}
			else if ("export".equals(cmd))
			{
				try
				{
					int numNews = (Integer)job.getValue("numNews");
					exportNews(numNews);
				}
				catch (InvalidDataException ex)
				{
					if (shouldFail)
					{
						response.setDone("Expected failure for command " + cmd + ": " + ex.getMessage());
					}
					else
					{
						response.setError("Unexpected failure for command " + cmd + ": " + ex.getMessage(), Arrays.asList(ex));			 
					}
					return;
				}
			}
			if (shouldFail)
			{
				response.setError("The permission test plugin executed succesfully, " +
					"but we expected a failure for command: " + cmd,
					null);
			}
			else
			{
				response.setDone("The permission test plugin executed succesfully for command: " + cmd);
			}
		}
		catch (BaseException ex)
		{
			response.setError("Unexpected failure: " + ex.getMessage(), Arrays.asList(ex));			 
		}
	}
		
	private void createNews(String description)
	{
		DbControl dc = sc.newDbControl();
		try
		{
			News n = News.getNew(dc, new Date(), new Date());
			n.setName("Permission test plugin");
			n.setDescription(description);
			dc.saveItem(n);
			dc.commit();
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	private void exportNews(int numNews)
	{
		DbControl dc = sc.newDbControl();
		try
		{
			List<News> news = new ArrayList<News>(News.getQuery().list(dc));
			if (news.size() != numNews) 
			{
				throw new InvalidDataException("Not allowed to read all news; expected " + numNews+
					"; got " + news.size());
			}
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}
