/*
	$Id: TestDbControl.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
import net.sf.basedb.core.*;

public class TestDbControl
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing DbControl");
		test_weak_reference();
		test_logout();

		int roleId = TestRole.test_create(true);
		test_detach(roleId);
		test_disconnect(roleId);
		TestRole.test_delete(roleId);

		write("++Testing DbControl "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}
	
	static void test_weak_reference()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, SystemItems.getId(User.ROOT));
			dc = null; // Now we only have a weak reference to the DbControl object from the User object
			write("--NOTE! The warning message about an unclosed DbControl is expected.");
			System.gc();  // Should clean up the weak reference
			if (!u.isDetached())
			{
				throw new BaseException("Expected the item to be detached");
			}
		}
		catch (PermissionDeniedException ex)
		{
			write("--Test weak reference OK");
		}
		catch (Throwable ex)
		{
			write("--Test weak reference FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_logout()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, SystemItems.getId(User.ROOT));
			TestUtil.logout(); // Should close all open DbControl objects
			dc.commit(); // Should throw ConnectionClosedException
			throw new BaseException("Expected ConnectionClosedException");
		}
		catch (ConnectionClosedException ex)
		{
			write("--Test logout OK");
		}
		catch (Throwable ex)
		{
			write("--Test logout FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
			try
			{
				TestUtil.login();
			}
			catch (Throwable ex)
			{}
		}
	}

	static void test_detach(int roleId)
	{
		if (roleId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getById(dc, roleId);
			TestRole.write_item(0, r);
			
			dc.detachItem(r);
			dc.close();
			r.setName("Detached role");
			dc = TestUtil.getDbControl();
			dc.reattachItem(r);
			dc.commit();
			dc.close();
			
			dc = TestUtil.getDbControl();
			r = Role.getById(dc, roleId);
			TestRole.write_item(1, r);
			if (!"Detached role".equals(r.getName())) throw new BaseException("Role.getName() != Detached role");
			write("--Test detach OK");
		}
		catch (Throwable ex)
		{
			write("--Test detach FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_disconnect(int roleId)
	{
		if (roleId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getById(dc, roleId);
			TestRole.write_item(0, r);
			
			dc.disconnect();
			if (dc.isConnected())
			{
				throw new BaseException("DbControl is still connected");
			}
			r.setName("Disconnected role");
			dc.reconnect();
			dc.commit();
			
			dc = TestUtil.getDbControl();
			r = Role.getById(dc, roleId);
			TestRole.write_item(1, r);
			if (!"Disconnected role".equals(r.getName())) throw new BaseException("Role.getName() != Disconnected role");
			write("--Test disconnect OK");
		}
		catch (Throwable ex)
		{
			write("--Test disconnect FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write(String message)
	{
		System.out.println(message);
	}

}

