import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.HardwareType;
import net.sf.basedb.core.ItemResultList;
import net.sf.basedb.core.SystemItems;

/*
 $Id$

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

 BASE is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 3
 of the License, or (at your option) any later version.

 BASE is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA  02111-1307, USA.
 */

public class TestHardwareType
{

	static boolean ok = true;
	
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing hardware type");
		write_header();
		// Standard tests: load and list
		test_load(SystemItems.getId(HardwareType.SCANNER));
		test_list();

		write("++Testing hardware type "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			HardwareType ht = HardwareType.getById(dc, id);
			write_item(0, ht);
			write("--Load hardware type OK");
		}
		catch (Throwable ex)
		{
			write("--Load hardware type FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<HardwareType> l = HardwareType.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			write("--List hardware types OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List hardware types FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription");
			write("-- \t-- \t--------- \t-----------");
		}
	}
	
	static void write_item(int i, HardwareType ht)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+ht.getId()+"\t"+ht.getName()+"\t"+
			ht.getDescription());
	}
	
	static void write(String message)
	{
		System.out.println(message);
	}
}
