/*
	$Id: TestProjectKey.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
import net.sf.basedb.core.*;
import java.util.EnumSet;

public class TestProjectKey
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing project key");
		write_header();
		// Extra test: create project for the key
		int projectId = TestProject.test_create(true);
		
		// Standard tests: create, load
		int id = test_create(projectId, EnumSet.of(Permission.WRITE));
		test_load(id);

		// Extra tests: create new key with same permission -> id2 == id
		int id2 = test_create(projectId, EnumSet.of(Permission.WRITE));
		write("--id==id2 "+(id==id2 ? "OK" : "FAILED id="+id+"; id2="+id2));
		ok = ok && (id == id2);

		// Extra tests: create new key with different permission -> id3 != id
		int id3 = test_create(projectId, EnumSet.of(Permission.USE));
		write("--id!=id3 "+(id!=id3 ? "OK" : "FAILED id="+id+"; id3="+id3));
		ok = ok && (id != id3);

		// Extra test: add another project -> id3 != id
// 		int projectId2 = TestProject.test_create();
// 		int id3 = test_addproject(id, project2_id);
// 		write("--id!=id3 "+(id != id3 ? "OK" : "FAILED id="+id+"; id3="+id3));
// 		ok = ok && (id != id3);
// 
// 		test_listprojects(id);
// 		test_listprojects(id3);
// 
// 		// Extra test: remove the project again -> id4 == id
// 		int id4 = test_removeproject(id3, project2_id);
// 		write("--id==id4 "+(id==id4 ? "OK" : "FAILED id="+id+"; id4="+id4));
// 		ok = ok && (id == id4);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();

		// Standard test: Delete
// 		test_delete(id);
// 		test_delete(id3);

		TestProject.test_delete(projectId);
//		TestProject.test_delete(projectId2);

		test_delete_unused();

		write("++Testing project key "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int projectId, EnumSet<Permission> permissions)
	{
		if (projectId == 0) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Project p = Project.getById(dc, projectId);
			ProjectPermissions pp = new ProjectPermissions();
			pp.setPermissions(p, permissions);
			ProjectKey pk = ProjectKey.getNewOrExisting(dc, pp);
			if (!pk.isInDatabase())
			{
				dc.saveItem(pk);
				dc.commit();
			}
			id = pk.getId();
			write_item(0, pk);
			write("--Create project key OK");
		}
		catch (Throwable ex)
		{
			write("--Create project key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ProjectKey pk = ProjectKey.getById(dc, id);
			write_item(0, pk);
			write("--Load project key OK");
		}
		catch (Throwable ex)
		{
			write("--Load project key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete_unused()
	{
		try
		{
			ProjectKey.deleteUnusedKeys();
			write("--Delete unused project keys OK");
		}
		catch (Throwable ex)
		{
			write("--Delete unused project keys FAILED");
			ex.printStackTrace();
			ok = false;
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID");
			write("-- \t--");
		}
	}
	static void write_item(int i, ProjectKey pk)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+pk.getId());
	}
	static void write_item(int i, Project p, EnumSet<Permission> permissions)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+p.getId()+"\t"+p.getName()+"\t"+p.getDescription()+"\t"+permissions);
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	/*
	static int test_addproject(int id, int projectId)
	{
		if (id == 0 || project_id == 0) return 0;
		int new_id = 0;
		BaseControl bc = null;
		try
		{
			bc = TestUtil.getBaseControl();
			ProjectKey pk = ProjectKey.getById(bc, id);
			Project p = Project.getById(bc, project_id);
			pk.setPermission(p, PERMISSION.ALL);
			pk.save();
			bc.commit();
			new_id = pk.getId();
			write("--Add project to project key OK");
		}
		catch (Throwable ex)
		{
			write("--Add project to project key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (bc != null) bc.close();
		}
		return new_id;
	}

	static int test_removeproject(int id, int project_id)
	{
		if (id == 0 || project_id == 0) return 0;
		int new_id = 0;
		BaseControl bc = null;
		try
		{
			bc = TestUtil.getBaseControl();
			ProjectKey pk = ProjectKey.getById(bc, id);
			Project p = Project.getById(bc, project_id);
			pk.setPermission(p, PERMISSION.DENIED);
			pk.save();
			bc.commit();
			new_id = pk.getId();
			write("--Remove project from project key OK");
		}
		catch (Throwable ex)
		{
			write("--Remove project from project key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (bc != null) bc.close();
		}
		return new_id;
	}

	static void test_listprojects(int id)
	{
		if (id == 0) return;
		BaseControl bc = null;
		try
		{
			bc = TestUtil.getBaseControl();
			ProjectKey pk = ProjectKey.getById(bc, id);
//			List l = Project.find(bc, null);
			List l = pk.getProjectList(null);
			for (int i = 0; i<l.size(); i++)
			{
				Project p = (Project)l.get(i);
				write_item(i, p, pk.getPermission(p));
			}
			write("--List projects in project key OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List projects in project key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (bc != null) bc.close();
		}
	}
*/
}

