/*
	$Id: TestRole.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2005 Samuel Andersson, Jari Hakkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
import net.sf.basedb.core.*;

import java.util.Date;
public class TestRole
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing roles");
		write_header();
		// Standard tests: create, load, list
		int id = test_create(true);
		int id2 = test_create(false);
		test_load(id);
		test_list(-1);

		// Extra tests: add, list and remove users
		test_listusers(id, 0);
		test_adduser(id, TestUtil.getSessionControl().getLoggedInUserId());
		test_listusers(id, 1);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_removeuser(id, TestUtil.getSessionControl().getLoggedInUserId());
		test_listusers(id, 0);

		// Standard test: Delete
		test_delete(id);
		test_delete(id2);
		write("++Testing roles "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.ROLE)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getNew(dc);
			if (setAll)
			{
				r.setName("Test role");
				r.setDescription("Added at "+new Date());
			}
			dc.saveItem(r);
			dc.commit();
			id = r.getId();
			write_item(0, r);
			write("--Create role OK");
		}
		catch (Throwable ex)
		{
			write("--Create role FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getById(dc, id);
			write_item(0, r);
			write("--Load role OK");
		}
		catch (Throwable ex)
		{
			write("--Load role FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<Role> l = Role.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List roles OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List roles FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getById(dc, id);
			dc.deleteItem(r);
			dc.commit();
			write("--Delete role OK");
		}
		catch (Throwable ex)
		{
			write("--Delete role FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			System.out.println("   \tID \tName      \tDescription");
			System.out.println("-- \t-- \t--------- \t-----------");
		}
	}
	static void write_item(int i, Role r)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+r.getId()+"\t"+r.getName()+"\t"+r.getDescription());
	}
	static void write_item(int i, User u)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+u.getId()+"\t"+u.getName()+"\t"+u.getDescription());
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_adduser(int id, int userId)
	{
		if (id == 0 || userId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getById(dc, id);
			User u = User.getById(dc, userId);
			r.addUser(u);
			dc.commit();
			write("--Add user to role OK");
		}
		catch (Throwable ex)
		{
			write("--Add user to role FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_listusers(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getById(dc, id);
			ItemResultList<User> l = r.getUsers().list(dc);
			for (int i=0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List users in role OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List users in role FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_removeuser(int id, int userId)
	{
		if (id == 0 || userId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Role r = Role.getById(dc, id);
			User u = User.getById(dc, userId);
			r.removeUser(u);
			dc.commit();
			write("--Remove user from role OK");
		}
		catch (Throwable ex)
		{
			write("--Remove user from role FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}

