/*
	$Id: TestRoleKey.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Hakkinen

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
import net.sf.basedb.core.*;
import java.util.EnumSet;
import java.util.List;
import java.util.Set;
public class TestRoleKey
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing role key");
		write_header();
		// Standard tests: load, list
		int id = test_load(Item.USER);

		// Extra tests: add, list and remove roles/permissions
		test_listroles(id);
		test_addrole(id, SystemItems.getId(Role.GUEST), EnumSet.of(Permission.WRITE));
		test_listroles(id);
		
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		test_removerole(id, SystemItems.getId(Role.GUEST));
		test_listroles(id);

		write("++Testing role key "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}


	static int test_load(Item itemType)
	{
		if (!TestUtil.hasPermission(Permission.READ, Item.ROLE)) return 0;
		DbControl dc = null;
		int id = 0;
		try
		{
			dc = TestUtil.getDbControl();
			RoleKey rk = RoleKey.getByItemType(dc, itemType);
			write_item(0, rk);
			write("--Load role key OK");
			id = rk.getId();
		}
		catch (Throwable ex)
		{
			write("--Load role key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}


	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tItem type\tDescription");
			write("-- \t-- \t--------- \t---------\t-----------");
		}
	}
	static void write_item(int i, RoleKey rk)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+rk.getId()+"\t"+rk.getName()+"\t"+rk.getItemType()+"\t"+rk.getDescription());
	}
	static void write_item(int i, Role r, Set<Permission> permissions)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+r.getId()+"\t"+r.getName()+"\t"+r.getDescription()+"\t"+permissions);
	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_addrole(int id, int roleId, EnumSet<Permission> permissions)
	{
		if (id == 0 || roleId == 0 || !TestUtil.hasPermission(Permission.WRITE, Item.ROLE)) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			RoleKey rk = RoleKey.getById(dc, id);
			Role r = Role.getById(dc, roleId);
			rk.setPermissions(r, permissions);
			dc.commit();
			write("--Add role to role key OK");
		}
		catch (Throwable ex)
		{
			write("--Add role to role key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_listroles(int id)
	{
		if (id == 0 || !TestUtil.hasPermission(Permission.READ, Item.ROLE)) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			RoleKey rk = RoleKey.getById(dc, id);
			// List l = Role.find(bc, null);
			List<Role> l = Role.getQuery().list(dc);
			for (int i=0; i<l.size(); i++)
			{
				Role r = l.get(i);
				write_item(i, r, rk.getPermissions(r));
			}
			write("--List roles in role key OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List roles in role key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_removerole(int id, int roleId)
	{
		if (id == 0 || roleId == 0 || !TestUtil.hasPermission(Permission.WRITE, Item.ROLE)) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			RoleKey rk = RoleKey.getById(dc, id);
			Role r = Role.getById(dc, roleId);
			rk.setPermissions(r, EnumSet.noneOf(Permission.class));
			dc.commit();
			write("--Remove role from role key OK");
		}
		catch (Throwable ex)
		{
			write("--Remove role from role key FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}


}

