/**
	$Id: PowerUserTest.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA. 
*/
package net.sf.basedb.test.roles;

import java.util.Arrays;
import java.util.EnumSet;
import java.util.List;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.ArrayBatch;
import net.sf.basedb.core.ArrayDesign;
import net.sf.basedb.core.ArraySlide;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.FileType;
import net.sf.basedb.core.Group;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.HardwareType;
import net.sf.basedb.core.IntegerParameterType;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Permission;
import net.sf.basedb.core.Plate;
import net.sf.basedb.core.PlateGeometry;
import net.sf.basedb.core.PlateType;
import net.sf.basedb.core.Platform;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.Project;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.ProtocolType;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.core.RawDataTypes;
import net.sf.basedb.core.Software;
import net.sf.basedb.core.SoftwareType;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.SystemItems;
import net.sf.basedb.core.Type;
import net.sf.basedb.test.FileUtil;
import net.sf.basedb.test.MouseData;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;

/**
	This class contains the tests for the power user created
	by the root user.
	<ol>
	<li>Create a project {@link #createProject(DbControl, String)} and activate it
	<li>Create several protocols: {@link #createProtocol(DbControl, String, String)}
	<li>Create hardware: {@link #createHardware(DbControl, String, String)}
	<li>Create software: {@link #createSoftware(DbControl, String, String)}
	<li>Create annotation types: {@link #createAnnotationType(DbControl, String, Type, Object[], Item[])}
	<li>Create plate type: {@link #createPlateType(DbControl, String, int, int)}
	<li>Create file formats for import:
	<li>Import plates:
	<li>Create array design: {@link #createArrayDesign(DbControl, Platform, String)}
	<li>Import features:
	<li>Create array batch: {@link #createArrayBatch(DbControl, String, ArrayDesign, Hardware, Protocol)}
	<li>Create array slides: {@link #createArraySlide(DbControl, String, ArrayBatch, int)}
	</ol>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-05 17:16:58 +0200 (Fri, 05 Sep 2008) $
*/
public class PowerUserTest
{

	public static boolean ok = true;
	
	/**
		Run the tests in this class.
	*/
	public static boolean run()
	{
		DbControl dc = null;
		TestUtil.write("++Entering power user test\n");
		try
		{
			TestUtil.login("power", "power", false);
			
			// Project
			dc = TestUtil.getDbControl();
			Project p = createProject(dc, "Project A");
			dc.commit();
			TestUtil.getSessionControl().setActiveProject(p);
			
			// Protocols, hardware and software
			dc = TestUtil.getDbControl();
			Protocol sampling = createProtocol(dc, "Sampling A", ProtocolType.SAMPLING);
			Protocol extraction = createProtocol(dc, "Extraction A", ProtocolType.EXTRACTION);
			Protocol labeling = createProtocol(dc, "Labeling A", ProtocolType.LABELING);
			Protocol hybridization = createProtocol(dc, "Hybridization A", ProtocolType.HYBRIDIZATION);
			Protocol scanning = createProtocol(dc, "Scanning A", ProtocolType.SCANNING);
			Protocol feature = createProtocol(dc, "Feature extraction A", ProtocolType.FEATURE_EXTRACTION);
			Protocol printing = createProtocol(dc, "Printing A", ProtocolType.PRINTING);
			
			Hardware hybStation = createHardware(dc, "Hybridization station A", HardwareType.HYBRIDIZATION_STATION);
			Hardware robot = createHardware(dc, "Print robot A", HardwareType.PRINT_ROBOT);
			Hardware scanner = createHardware(dc, "Scanner A", HardwareType.SCANNER);
			Software software = createSoftware(dc, "Software A", SoftwareType.FEATURE_EXTRACTION);
			
			AnnotationType drugResistance = createAnnotationType(dc, "Drug resistance", Type.STRING, 
					new String[] { "high", "medium", "low" },  new Item[] { Item.BIOSOURCE }
				);
			AnnotationType hours = createAnnotationType(dc, "Time (hours)", Type.INT, null, 
					new Item[] { Item.SAMPLE } 
				);
			
			AnnotationType dyeSwap = createAnnotationType(dc, "Dye swap", Type.BOOLEAN, null, 
					new Item[] { Item.RAWBIOASSAY } 
				);
			
			PlateType plateType = createPlateType(dc, "Plate type A", 16, 24);
			
			PluginConfiguration platesImporter = PluginUtil.createPluginConfiguration(dc, "Plates for project A", 
					"net.sf.basedb.plugins.PlateFlatFileImporter", MouseData.getPlateParameters());

			PluginConfiguration genePixFeatureImporter = PluginUtil.createPluginConfiguration(dc, "Features from GenePix file",
					"net.sf.basedb.plugins.ReporterMapFlatFileImporter", MouseData.getGenePixReporterMapParameters());
			
			PluginDefinition printMapImporter = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.PrintMapFlatFileImporter");

			PluginConfiguration rawDataImporter = PluginUtil.createPluginConfiguration(dc, "Raw data for project A", 
					"net.sf.basedb.plugins.RawDataFlatFileImporter", MouseData.getGenePixRawDataParameters(false));
			Util.setAnnotation(rawDataImporter, dyeSwap, false);
			
			PluginConfiguration rawDataImporterDyeSwap = PluginUtil.createPluginConfiguration(dc, "Raw data for project A (dye-swap)", 
					"net.sf.basedb.plugins.RawDataFlatFileImporter", MouseData.getGenePixRawDataParameters(true));
			Util.setAnnotation(rawDataImporterDyeSwap, dyeSwap, true);
			
			dc.commit();
			
			dc = TestUtil.getDbControl();

			File plates = FileUtil.uploadFile(dc, "/mouse", "plates_and_reporters.mouse.v4.37k.txt", FileType.PLATE);			
			File printMap = FileUtil.uploadFile(dc, "/mouse", "printmap.mouse.v4.37k.tam", FileType.PRINT_MAP);
			File cdfFile = FileUtil.uploadFile(dc, "/affymetrix/cdf", "MG_U74Av2.cdf.tar.bz2", FileType.REPORTER_MAP);
			dc.commit();
			
			importPlates(platesImporter, plates, "Plate A", plateType);
			
			dc = TestUtil.getDbControl();
			Platform genericPlatform = Platform.getByExternalId(dc, Platform.GENERIC);
			Platform affymetrixPlatform = Platform.getByExternalId(dc, Platform.AFFYMETRIX);
			ArrayDesign design = createArrayDesign(dc, genericPlatform, "Array design A");
			FileUtil.addDataFile(dc, design, DataFileType.GENERIC_PRINT_MAP, printMap, true, true);
			ArrayDesign affyDesign = createArrayDesign(dc, affymetrixPlatform, "Affymetrix A");
			FileUtil.addDataFile(dc, affyDesign, DataFileType.AFFYMETRIX_CDF, cdfFile, true, true);
			dc.commit();

			importFeatures(printMapImporter, printMap, design);
			
			dc = TestUtil.getDbControl();
			RawDataType genepix = RawDataTypes.getSafeRawDataType("genepix");
			p = Project.getById(dc, p.getId());
			p.setDefaultItem(dc, genericPlatform, Project.Default.PLATFORM);
			p.setDefaultRawDataType(genepix);
			p.setDefaultItem(dc, design, Project.Default.ARRAYDESIGN);
			p.setDefaultItem(dc, software, Project.Default.SOFTWARE);
			p.setDefaultItem(dc, hybStation, Project.Default.HYBRIDIZATION_HARDWARE);
			p.setDefaultItem(dc, scanner, Project.Default.SCANNER_HARDWARE);
			p.setDefaultItem(dc, robot, Project.Default.PRINTROBOT_HARDWARE);
			p.setDefaultItem(dc, scanning, Project.Default.SCANNING_PROTOCOL);
			p.setDefaultItem(dc, sampling, Project.Default.SAMPLING_PROTOCOL);
			p.setDefaultItem(dc, extraction, Project.Default.EXTRACTING_PROTOCOL);
			p.setDefaultItem(dc, labeling, Project.Default.LABELING_PROTOCOL);
			p.setDefaultItem(dc, hybridization, Project.Default.HYBRIDIZATION_PROTOCOL);
			p.setDefaultItem(dc, feature, Project.Default.FEATURE_EXTRACTION_PROTOCOL);
			p.setDefaultItem(dc, printing, Project.Default.PRINTING_PROTOCOL);	
			
			ArrayBatch batch = createArrayBatch(dc, "Array batch A", design, robot, printing);
			ArraySlide slide1 = createArraySlide(dc, "Array slide A.1", batch, 1);
			ArraySlide slide2 = createArraySlide(dc, "Array slide A.2", batch, 2);
			ArraySlide slide3 = createArraySlide(dc, "Array slide A.3", batch, 3);
			ArraySlide slide4 = createArraySlide(dc, "Array slide A.4", batch, 4);
			
			// Affymetrix
			ArrayBatch affyBatch = createArrayBatch(dc, "Affymetrix batch A", affyDesign, null, null);
			ArraySlide affySlide1 = createArraySlide(dc, "Affymetrix slide A.1", affyBatch, 1);
			ArraySlide affySlide2 = createArraySlide(dc, "Affymetrix slide A.2", affyBatch, 2);
			ArraySlide affySlide3 = createArraySlide(dc, "Affymetrix slide A.3", affyBatch, 3);
			dc.commit();

			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving power user test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}
	
	/**
		Create a project.
	*/
	public static Project createProject(DbControl dc, String name)
	{
		TestUtil.write("--Creating project: " + name + "\n");
		
		Project p = Project.getNew(dc);
		p.setName("Project A");
		
		Group groupA = Util.findGroup(dc, "Group A");
		
		p.setPermissions(groupA, EnumSet.of(Permission.USE));
		dc.saveItem(p);
		return p;
	}

	/**
		Create a protocol.
	*/
	public static Protocol createProtocol(DbControl dc, String name, String type)
	{
		TestUtil.write("--Creating protocol: " + name + "\n");
		ProtocolType pt = ProtocolType.getById(dc, SystemItems.getId(type));
		Protocol p = Protocol.getNew(dc, pt);
		p.setName(name);
		dc.saveItem(p);
		return p;
	}
	
	/**
		Create a hardware.
	*/
	public static Hardware createHardware(DbControl dc, String name, String type)
	{
		TestUtil.write("--Creating hardware: " + name + "\n");
		HardwareType ht = HardwareType.getById(dc, SystemItems.getId(type));
		Hardware hw = Hardware.getNew(dc, ht);
		hw.setName(name);
		dc.saveItem(hw);
		return hw;
	}

	/**
		Create a software.
	*/
	public static Software createSoftware(DbControl dc, String name, String type)
	{
		TestUtil.write("--Creating software: " + name + "\n");
		SoftwareType st = SoftwareType.getById(dc, SystemItems.getId(type));
		Software sw = Software.getNew(dc, st);
		sw.setName(name);
		dc.saveItem(sw);
		return sw;
	}
	
	/**
		Create an annotation type.
	*/
	public static AnnotationType createAnnotationType(DbControl dc, String name, Type type, Object[] values, Item[] items)
	{
		TestUtil.write("--Creating annotation type: " + name + "\n");
		
		AnnotationType at = AnnotationType.getNew(dc, type);
		at.setName(name);
		if (items != null)
		{
			for (int i = 0; i < items.length; ++i)
			{
				at.enableForItem(items[i]);
			}
		}
		if (values != null)
		{
			at.setEnumeration(true);
			at.setDisplayAsList(false);
			at.setValues(Arrays.asList(values));
		}
		dc.saveItem(at);
		return at;
	}

	/**
		Create a plate type.
	*/
	public static PlateType createPlateType(DbControl dc, String name, int rows, int columns)
	{
		TestUtil.write("--Creating plate type: " + name + "\n");
		
		PlateGeometry geometry = Util.findGeometry(dc, rows, columns);
		PlateType pt = PlateType.getNew(dc, geometry);
		pt.setName(name);
		dc.saveItem(pt);
		return pt;
	}
	
	/**
		Create an array design.
	*/
	public static ArrayDesign createArrayDesign(DbControl dc, Platform platform, String name)
	{
		TestUtil.write("--Creating array design: " + name + "\n");
		ArrayDesign design = ArrayDesign.getNew(dc, platform);
		design.setName(name);
		
		if (!platform.isFileOnly())
		{
			List<Plate> plates = Util.findPlates(dc);
			design.setPlateList(plates);
		}
		
		dc.saveItem(design);
		return design;
	}

	/**
		Create an array batch.
	*/
	public static ArrayBatch createArrayBatch(DbControl dc, String name, ArrayDesign design, Hardware robot, Protocol protocol)
	{
		TestUtil.write("--Creating array batch: " + name + "\n");
		ArrayBatch batch = ArrayBatch.getNew(dc, design);
		batch.setName(name);
		batch.setPrintRobot(robot);
		batch.setProtocol(protocol);
		dc.saveItem(batch);
		return batch;
	}

	/**
		Create an array slide.
	*/
	public static ArraySlide createArraySlide(DbControl dc, String name, ArrayBatch batch, int index)
	{
		TestUtil.write("--Creating array slide: " + name + "\n");
		ArraySlide slide = ArraySlide.getNew(dc, batch);
		slide.setName(name);
		slide.setBatchIndex(index);
		dc.saveItem(slide);
		return slide;
	}
	
	public static void importPlates(PluginConfiguration config, File file, String plateNamePrefix, PlateType plateType)
	{
		TestUtil.write("--Importing plates from file: " + file.getName() + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			config = PluginConfiguration.getById(dc, config.getId());
			Job job = Job.getNew(dc, config.getPluginDefinition(), config, null);
			job.setName("Importing plates for project A");
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("plateType", new ItemParameterType<PlateType>(PlateType.class, null), plateType);
			job.setParameterValue("plateNamePrefix", new StringParameterType(), plateNamePrefix);
			job.setParameterValue("namePadding", new IntegerParameterType(), 4);

			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
		
	}
	
	/**
		Import features.
	*/
	public static void importFeatures(PluginDefinition plugin, File file, ArrayDesign design)
		throws Exception
	{
		TestUtil.write("--Importing features from file: " + file.getName() + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{
			plugin = PluginDefinition.getById(dc, plugin.getId());
			design.setPlateList(Util.findPlates(dc));
			Job job = Job.getNew(dc, plugin, null, null);
			job.setName("Importing features to array design: " + design.getName());
			job.setParameterValue("file", new FileParameterType(), file);
			job.setParameterValue("arrayDesign", 
				new ItemParameterType<ArrayDesign>(ArrayDesign.class, null), design);
			dc.saveItem(job);
			dc.commit();
			PluginUtil.executeJob(job);
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
}
