/**
	$Id: AnnotationTypeClient.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Martin Svensson, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.ws.client;

import net.sf.basedb.info.AnnotationTypeInfo;
import net.sf.basedb.info.QueryOptions;

import org.apache.axis2.AxisFault;

/**
 	Web service client for annotation types.
 	Calls services in {@link net.sf.basedb.ws.server.AnnotationTypeService}
  	@author Nicklas
  	@version 2.5
 */
public class AnnotationTypeClient 
	extends AbstractRPCClient 
{
	private final SessionClient session;
	
	/**
	 	Creates new AnnotationTypeClient object for an active session client.
	  	@param session The active session client
	 */
	public AnnotationTypeClient(SessionClient session)
	{
		super(session.getUrl(), "AnnotationType", session.getServiceFactory());
		this.session = session;
	}
	
	/**
		Get info about an annotation type when you know the ID.
		@param itemId The ID of the annotation type
		@return An AnnotationTypeInfo object
	  	@throws AxisFault If communication with web service fails.
	*/
	public AnnotationTypeInfo getById(int itemId)
		throws AxisFault
	{
		return invokeBlocking("getById", AnnotationTypeInfo.class, 
			session.getId(), itemId);
	}
	/**
	 	Gets info about each projects that are available for the active session client.
	 	@param itemType The name of the item type as defined by {@link net.sf.basedb.core.Item#name()}
	 		or null to return annotation types for all item types
	  	@param qOpt Options to put on the query 
	  	@return An array with a info object for each annotation type
	  	@throws AxisFault If communication with web service fails.
	 */
	public AnnotationTypeInfo[] getAnnotationTypes(String itemType, QueryOptions qOpt)
		throws AxisFault
	{		 
		AnnotationTypeInfo[] result = invokeBlocking("getAnnotationTypes", AnnotationTypeInfo[].class, 
			session.getId(), itemType, qOpt);
		return result;
	}
}
