/**
	$Id: ReporterClient.java 4480 2008-09-05 15:16:58Z jari $

	Copyright (C) 2007 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/
package net.sf.basedb.ws.client;

import net.sf.basedb.info.ExtendedPropertyInfo;
import net.sf.basedb.info.QueryOptions;

import java.io.IOException;
import java.io.InputStream;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.client.Options;
import org.apache.axis2.rpc.client.RPCServiceClient;

/**
 	Web service client for reporter.
  	@author Martin
  	@since 2.5
 	
 */
public class ReporterClient 
	extends AbstractRPCClient
{
	private SessionClient session;
	
	/**
	 	Creates a new instance of ReporterClient used together with a SessionClient
	  	@param session The SessionClient to use.
	 */
	public ReporterClient(SessionClient session)
	{
		super(session.getUrl(), "Reporter", session.getServiceFactory());
		this.session = session;
	}
	
	/**
	 	Gets the extended properties, for reporters, through web services. 
	  	@return An array with ExtendedPropertyInfo
	  	@throws AxisFault
	 */
	public ExtendedPropertyInfo[] getExtendedProperties()
		throws AxisFault
	{
		ExtendedPropertyInfo[] result = invokeBlocking("getExtendedProperties", ExtendedPropertyInfo[].class, session.getId());
		return result;
	}

	/**
	 	Gets an input stream for a file with info about 
	 	reporters in a reporter list. 
	  	@param reporterListId Id of the reporter list to export and download.
	  	@param extPropInfo An array of ExtendedPropertyInfo objects to include in the export/download.
	  	@param qOpt QueryOptions to use when selecting items from the database.
	  	@return InputStream object to the exported data.
	  	@throws IOException If something goes wrong with the input stream
	 */
	public InputStream downloadReportersInList(int reporterListId, ExtendedPropertyInfo[] extPropInfo, QueryOptions qOpt)
		throws IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		options.setTimeOutInMilliSeconds(10000);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadReportersInList", 
				session.getId(), 
				reporterListId,
				extPropInfo,
				qOpt
			);
	}
	
	/**
	 	Downloads exported reporter properties to an input stream. 
	  	@param extPropInfo A list with the extended properties to export.
	  	@param qOpt Options on the query that selects data from the database.
	  	@return An input stream to a text file.
	  	@throws IOException If something goes wrong when getting the exported properties.
	 */
	public InputStream downloadReporters(ExtendedPropertyInfo[] extPropInfo, QueryOptions qOpt)
		throws IOException
	{
		RPCServiceClient service = getService();
		Options options = service.getOptions();		
		options.setProperty(Constants.Configuration.ENABLE_MTOM, Constants.VALUE_TRUE);
		options.setTimeOutInMilliSeconds(10000);
		service.setOptions(options);
		return invokeFileBlocking
			(
				"downloadReporters", 
				session.getId(),
				extPropInfo,
				qOpt
			);
	}

}
